SystemOrganization addCategory: #'SWE-IssueTracker'!

GlorpSession subclass: #ITGlorpSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession commentStamp: 'mt 12/15/2007 10:12' prior: 0!
I'm a usual GlorpSession and control a cache manager, a description system and all unit of works. Every manipulation of the SQL database goes through my "fingers".

Use #inUnitOfWorkDo: for watching over "dirty" objects and change all relevant data in the database. But DO NOT forget to register them with #register inside a unit of work.

Any read of data doesn't have to be embedded in a unit of work. Just use the result of #read:, #readOneOf:, #readManyOf:, #read:where: a. s. o.

If you want to delete an object you doesn't have to use a unit of work either.!

ITGlorpSession class
	instanceVariableNames: 'instance'!

!ITGlorpSession class methodsFor: 'accessing' stamp: 'afk 12/15/2007 13:10'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITGlorpSession"
	
	instance ifNil:
		[ instance := ITGlorpDB connect.].
	instance isLoggedIn ifFalse:
		[ instance := ITGlorpDB connect.].
	^ instance! !

!ITGlorpSession class methodsFor: 'instance creation' stamp: 'mt 12/26/2007 10:52'!
new
	Error signal: 'Use getInstance because', self name, ' is a singleton.'! !

TestCase subclass: #ITTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITTest subclass: #ITTestGlorpDB
	instanceVariableNames: 'login accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestGlorpDB methodsFor: 'testing' stamp: 'afk 12/13/2007 14:52'!
setUp
	
	super setUp.
	login := ITGlorpDB login.
	accessor := DatabaseAccessor forLogin: login.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:47'!
testConnect
	"Tests whether ITGlorpDB>>#connect answers with a valid GlorpSession which could be used for operating with the SQL database."
	
	| session |
	session := ITGlorpDB connect.
	
	self assert: (login username = session currentLogin username) description: 'Username of session does not equal login username'.
	self assert: (login connectString = session currentLogin connectString) description: 'ConnectString of session does not equal login ConnectString'.
	self assert: (login password = session currentLogin password) description: 'Password of session does not equal login password '.
	
	self assert: session isLoggedIn.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mt 12/14/2007 15:44'!
testLogin
	"Tests whether it is possible to use the default login for connecting to a vaild SQL database which is used in the whole application."
	
	self assert: accessor isLoggedIn not.
	accessor loginIfError: [:exception | self assert: false.].
	self assert: accessor isLoggedIn.
	accessor logout.
	self assert: accessor isLoggedIn not.! !

!ITTestGlorpDB methodsFor: 'testing' stamp: 'mig 12/20/2007 18:53'!
testUniqueSession
	"Tests whether a glorp session really is a singleton."


	self assert: ITGlorpSession getInstance notNil description: 'Glorp session is nil.'.
	self assert: (ITGlorpSession getInstance == ITGlorpSession getInstance) description: 'Glorp session is not unique.'.! !

ITTest subclass: #ITTestIssue
	instanceVariableNames: 'issue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	issue := ITIssue new.! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/8/2008 12:17'!
testAddComment
	"Tests whether it is possible to add a new ITIssueComment instance to an issue and retrieve this comment by its title again."

	self flag: #TODO. "Comments must be created with a user!!"	
	issue addComment: (ITIssueComment with: 'TESTCOMMENT').
	
	self 
		assert: issue comments last content = 'TESTCOMMENT' 
		description: 'A just added comment could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testComments
	"Tests whether the accessors for #comments do what they should do - set and get."
	
	| comments |
	comments := OrderedCollection new.
	issue comments: comments.
	
	self 
		assert: issue comments == comments 
		description: 'An ordered collection of comments was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:32'!
testDetails
	"Tests whether the accessors for #details do what they should do - set and get."
	
	| details |
	details := OrderedCollection new.
	issue details: details.
	
	self 
		assert: issue details == details 
		description: 'An ordered collection of details was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/8/2008 12:23'!
testGetTag
	"Tests whether it is possible to retrieve a detail which was just added to an issue."
	
	| detail |
	detail := ITIssueDetail new.
	detail tag: 'title'.
	detail content: 'TESTTITLE'.
	issue details: detail asOrderedCollection.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A just added detail could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 12/14/2007 16:30'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	issue id: 1.
	
	self
		assert: issue id = 1 
		description: 'An id was just set and could not be retrieved from the issue.'! !

!ITTestIssue methodsFor: 'testing' stamp: 'mt 1/8/2008 12:24'!
testSetDetailWithTo
	"Tests whether it is possible to add a new detail to an issue."
	
	issue setDetailWith: 'title' to: 'TESTTITLE'.
	
	self 
		assert: (issue getTag: 'title') = 'TESTTITLE' 
		description: 'A new detail was set and could not be retrieved from the issue.'! !

ITTest subclass: #ITTestIssueComment
	instanceVariableNames: 'comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	comment := ITIssueComment new.! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	comment content: 'TESTCOMMENT'.
	
	self 
		assert: comment content = 'TESTCOMMENT' 
		description: 'A content was just set and could not be retrieved from the comment.'! !

!ITTestIssueComment methodsFor: 'testing' stamp: 'mt 12/14/2007 16:29'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	comment id: 1.
	
	self 
		assert: comment id = 1 
		description: 'An id was just set and could not be retrieved from the comment.'! !

ITTest subclass: #ITTestIssueDetail
	instanceVariableNames: 'detail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	detail := ITIssueDetail new.! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'afk 1/9/2008 18:42'!
testContent
	"Tests whether the accessors for #content do what they should do - set and get."

	detail content: 'TESTCONTENT'.
	
	self 
		assert: detail content = 'TESTCONTENT' 
		description: 'A content was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:26'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	detail id: 1.
	
	self 
		assert: detail id = 1 
		description: 'An id was just set and could not be retrieved from the detail.'! !

!ITTestIssueDetail methodsFor: 'testing' stamp: 'mt 12/14/2007 16:27'!
testTag
	"Tests whether the accessors for #tag do what they should do - set and get."

	detail tag: 'TESTTAG'.
	
	self 
		assert: detail tag = 'TESTTAG' 
		description: 'A tag was just set and could not be retrieved from the detail.'! !

ITTest subclass: #ITTestIssueManager
	instanceVariableNames: 'issueManager securityManager testUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestIssueManager methodsFor: 'running' stamp: 'mt 1/8/2008 12:26'!
setUp
	
	super setUp.
	
	issueManager := ITIssueManager new.
	securityManager := ITSecurityManager new.
	
	testUser := ITUser new username: 'tmp_ITTestIssueManager'.
	securityManager addUser: testUser.
	! !

!ITTestIssueManager methodsFor: 'running' stamp: 'mig 1/8/2008 00:43'!
tearDown

	securityManager deleteUserWithID: testUser id.
	
	super tearDown.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/8/2008 12:27'!
testAddCommentAtIDBy
	"Tests whether it is possible to add a comment to an issue through the issue manager to the databse."

	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddCommentAtID'.

	issueManager addComment: 'tmp_testAddCommentAtID_comment' atID: issue id by: testUser.

	self 
		assert: (issueManager getIssueWithID: issue id) comments last content = 'tmp_testAddCommentAtID_comment'
		description: 'A comment to a new issue was just set and could not be retrieved through an issue manager.'.

	issueManager deleteIssueWithID: issue id.
! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/8/2008 12:33'!
testAddIssueWithTitle
	"Tests whether it is possible to add an issue with a title through the issue manager."
		
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testAddIssueWithTitle'.
	
	self 
		assert: (issue isKindOf: ITIssue) 
		description: 'ITIssueManager>>#addIssueWithTitle: is not returning an object with type ITIssue'.
		
	self
		deny: (issue id) isNil
		description: 'ITIssueManager>>#addIssueWithTitle: returns an ITIssue object which has no ID.'.
		
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testAddIssueWithTitle'
		description: 'A new issue with a title was just set and the title could not be retrieved through an issue manager.'.
	
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/8/2008 12:34'!
testDeleteIssueWithID
	"Tests whether it is possible to remove an issue with a given ID."

	| issue |
	
	issue := issueManager addIssueWithTitle: 'tmp_testDeleteIssueWithID'.
	
	issueManager deleteIssueWithID: issue id.
	
	self
		assert: (issueManager getIssueWithID: issue id) isNil
		description: 'A new issue was just added and could not be deleted through an issue manager.'
	

	! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/8/2008 12:35'!
testGetIssueWithID
	"Tests whether it is possible to retrieve an issue by its ID from the issue manager."
	
	| issue |
	issue := issueManager addIssueWithTitle: 'tmp_testGetIssueWithID'.
	
	self
		assert: ((issueManager getIssueWithID: issue id) getTag: 'title') = 'tmp_testGetIssueWithID'
		description: 'A new issue was just added and could not be retrieved through an issue manager'.
		
	issueManager deleteIssueWithID: issue id.! !

!ITTestIssueManager methodsFor: 'testing' stamp: 'mt 1/8/2008 12:36'!
testGetIssues
	"Tests whether the issue manager returns only objects with the type ITIssue which all have an ID."
	
	issueManager getIssues do: [ :issue |
		self 
			assert: (issue isKindOf: ITIssue)
			description: 'ITIssueManager>>#getIssues returns a collection in where not all objects are issues.'.
		
		self
			deny: (issue id) isNil
			description: 'ITIssueManager>>#getIssues returns a collection in where not all issues have an ID.'.
	].
	
	
	! !

ITTest subclass: #ITTestRole
	instanceVariableNames: 'role'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:39'!
setUp

	super setUp.
	role := ITRole new! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:20'!
testAddPrivilege
	"Tests whether it is possible to add a privilege but not twice the same one."
	
	2 timesRepeat: [role addPrivilege: 'write'].

	self
		assert: role privileges size = 1
		description: 'A privilege was added twice to a role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/8/2008 12:36'!
testAddUser
	"Tests whether it is possible to attach a user to a role."
	
	self flag: #TODO.! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:27'!
testHasPrivilege
	"Tests whether it is possible to add a privilege to a role and ask this role whether it has this privilege."
	
	role addPrivilege: 'write'.
		
	self
		assert: (role hasPrivilege: 'write')
		description: 'A privilege was just added and could not be retrieved from a role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:18'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	role id: 1.
	
	self
		assert: role id = 1 
		description: 'An id was just set and could not be retrieved from the role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:23'!
testPrivileges
	"Tests whether the accessors for #privileges do what they should do - set and get."

	| somePrivileges |
	somePrivileges := #('read' 'read' 'write' 'write').
	role privileges: somePrivileges.
	
	self
		assert: (role privileges isKindOf: Set) 
		description: 'The privileges of a role are not a set.'.
		
	self
		assert: role privileges = #('read' 'write') asSet
		description: 'Some privileges where just set to a role and could not be retrieved.' ! !

!ITTestRole methodsFor: 'testing' stamp: 'mig 1/7/2008 12:15'!
testRemovePrivilege
	"Tests whether it is possible to remove a privilege from a role."
	
	role
		addPrivilege: 'write';
		removePrivilege: 'write'.

	self
		assert: role privileges isEmpty
		description: 'A privilege was added and could not be removed from a role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/8/2008 12:36'!
testRemoveUser
	"Tests whether it is possible to detach a user from a role."
	
	self flag: #TODO.! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:15'!
testRolename
	"Tests whether the accessors for #rolename do what they should do - set and get."

	role rolename: 'testrole'.
	
	self
		assert: role rolename = 'testrole'
		description: 'A rolename was just set and could not be retrieved from the role.'! !

!ITTestRole methodsFor: 'testing' stamp: 'mt 1/8/2008 12:37'!
testUsers
	"Tests whether the accessors for #users do what they should do - set and get."
	
	self flag: #TODO.! !

ITTest subclass: #ITTestSecurityManager
	instanceVariableNames: 'securityManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 11:58'!
setUp

	super setUp.
	securityManager := ITSecurityManager new.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mig 1/9/2008 15:21'!
testAddRole
	"Tests whether it is possible to add a new role to the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testAddRole'.
	
	self 
		assert: (securityManager addRole: role) == role
		description: 'addRole does not return the just added object.'.
	
	self
		deny: role id isNil 
		description: 'New role could not be added to database through security manager because it got no ID.'.
		
	securityManager deleteRoleWithID: role id.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mig 1/9/2008 15:22'!
testAddUser
	"Tests whether it is possible to add a new user to the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testAddUser'.
	
	self
		assert: (securityManager addUser: user) == user
		description: 'addUser does not return the just added object.'.
	
	self
		deny: user id isNil 
		description: 'New user could not be added to database through security manager because it got no ID.'.
		
	securityManager deleteUserWithID: user id.! !

!ITTestSecurityManager methodsFor: 'tests' stamp: 'mig 1/9/2008 15:26'!
testDefaultUser
	"tests whether the default user is an ITUser object and has the correct name"

	self 
		assert: (securityManager defaultUser isKindOf: ITUser)
		description: 'Returned default user is not an ITUser object.'.
		
	self 
		assert: securityManager defaultUser username = 'Guest'
		description: 'Default username should be "Guest".'.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 12:56'!
testDeleteRoleWithID
	"Tests whether it is possible to remove a role with a given ID from the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testDeleteRoleWithID'.
	securityManager 
		addRole: role;
		deleteRoleWithID: role id.
		
	self
		assert: (securityManager getRoleWithID: role id) isNil
		description: 'A role could not be deleted through a security manager.'
	! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:10'!
testDeleteUserWithID
	"Tests whether it is possible to remove a user with a given ID from the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testDeleteUserWithID'.
	securityManager 
		addUser: user;
		deleteUserWithID: user id.
		
	self
		assert: (securityManager getUserWithID: user id) isNil
		description: 'A user could not be deleted through a security manager.'
	! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:40'!
testGetRole
	"Tests whether it is possible to retrieve a role instance with a given rolename from the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testGetRoleWithID'.
	securityManager addRole: role.
	
	self
		assert: ((securityManager getRole: role rolename) isKindOf: ITRole)
		description: 'A new role was just added to the database through a security manager and could not be retrieved by its name.'.
	
	securityManager deleteRoleWithID: role id.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 12:53'!
testGetRoleWithID
	"Tests whether it is possible to retrieve a role instance with a given ID from the database."
	
	|role|
	role := ITRole new.
	role rolename: 'tmp_testGetRoleWithID'.
	securityManager addRole: role.
	
	self
		assert: (securityManager getRoleWithID: role id) rolename = 'tmp_testGetRoleWithID'
		description: 'A new role was just added to the database through a security manager and could not be retrieved.'.
	
	securityManager deleteRoleWithID: role id.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:40'!
testGetUser
	"Tests whether it is possible to retrieve a user instance with a given username from the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testGetUser'.
	securityManager addUser: user.
	
	self
		assert: ((securityManager getUser: user username) isKindOf: ITUser)
		description: 'A new user was just added to the database through a security manager and could not be retrieved by its name.'.
	
	securityManager deleteUserWithID: user id.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:41'!
testGetUserWithID
	"Tests whether it is possible to retrieve a user instance with a given ID from the database."
	
	|user|
	user := ITUser new.
	user username: 'tmp_testGetUserWithID'.
	securityManager addUser: user.
	
	self
		assert: (securityManager getUserWithID: user id) username = 'tmp_testGetUserWithID'
		description: 'A new user was just added to the database through a security manager and could not be retrieved.'.
	
	securityManager deleteUserWithID: user id.! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:46'!
testHashAlgorithm
	"Tests whether Message Digest 5 Hash is used by the security manager."
	
	self
		assert: ITSecurityManager hashAlgorithm == MD5
		description: 'ITSecurityManager uses wrong hash algorithm. MD5 expected.'! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 13:50'!
testHashPassword
	"Tests whether a given password will be modified correctly through the security manager."
	
	self
		deny: (ITSecurityManager hashPassword: 'somepassword') = 'somepassword'
		description: 'A password was not hashed through security manager. The plain text was returned.'.
		
	self
		assert: (ITSecurityManager hashPassword: 'somepassword') = (ITSecurityManager hashPassword: 'somepassword')
		description: 'The security manager returns different hashes for the same plain text.'! !

!ITTestSecurityManager methodsFor: 'testing' stamp: 'mt 12/30/2007 14:04'!
testValidatePasswordFor
	"Tests whether security manager verifies the password for a user correctly."
	
	|user plaintext|
	plaintext := 'tmp_testValidatePasswordFor'.
	user := ITUser new.
	user username: 'tmp_testValidatePasswordFor_username'.
	user setPassword: plaintext.
	
	securityManager addUser: user.
	
	self
		assert: (securityManager validatePassword: plaintext for: user username)
		description: 'A correct password for a user could not be validated through the security manager.'.
		
	self
		deny: (securityManager validatePassword: (plaintext,'invalid_garbage') for: user username)
		description: 'An invalid password for a user was validated through the security manager. False expected.'.
		
	securityManager deleteUserWithID: user id.
! !

ITTest subclass: #ITTestSession
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:53'!
setUp

	super setUp.
	session := ITSession new.! !

!ITTestSession methodsFor: 'tests' stamp: 'mig 1/9/2008 16:45'!
testIsLoggedIn
	"tests whether isLoggedIn returns the correct values"
	
	self 
		deny: session isLoggedIn
		description: '#isLoggedIn must be false directly after its creation'.
	
	session user: ITUser new.
	
	self
		assert: session isLoggedIn
		description: '#isLoggedIn must be true directly after user was set'.
	
	self flag: #TODO. "test after logout"! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:41'!
testIssueManager
	"Tests whether there is only one issue manager per session."
	
	self
		assert: session issueManager == session issueManager
		description: 'An ITSession instance returns different ITIssueManager instances. There should only be one.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:44'!
testLanguage
	"Tests whether it is possible to set and get a language for translation issues."
	
	session language: #english.
	
	self
		assert: session language = #english
		description: 'A language was just set and could not be retrieved from a session.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:42'!
testSecurityManager
	"Tests whether there is only one security manager per session."
	
	self
		assert: session securityManager == session securityManager
		description: 'An ITSession instance returns different ITSecurityManager instances. There should only be one.'! !

!ITTestSession methodsFor: 'testing' stamp: 'mt 12/30/2007 11:52'!
testUser
	"Tests whether it is possible to set and get a user for a session."
	
	| user |
	user := ITUser new.
	
	session user: user.
	
	self
		assert: session user == user
		description: 'A user was just set and could not be retrieved from a session.'! !

ITTest subclass: #ITTestTag
	instanceVariableNames: 'tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestTag methodsFor: 'testing' stamp: 'afk 1/9/2008 18:40'!
setUp

	super setUp.
	tag := ITTag new.! !

!ITTestTag methodsFor: 'testing' stamp: 'afk 1/9/2008 19:05'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."
	
	tag id: 1.
	
	self 
		assert: tag id = 1 
		description: 'An id was just set and could not be retrieved from the tag.'! !

!ITTestTag methodsFor: 'testing' stamp: 'afk 1/9/2008 19:05'!
testTagDescription
	"Tests whether the accessors for #tagDescription do what they should do - set and get."
	
	tag description: 'TESTTAG'.
	
	self 
		assert: tag description = 'TESTTAG' 
		description: 'A desription was just set and could not be retrieved from the tag.'! !

!ITTestTag methodsFor: 'testing' stamp: 'afk 1/9/2008 19:05'!
testTagName
	"Tests whether the accessors for #tagName do what they should do - set and get."
	
	tag name: 'TESTTAG'.
	
	self 
		assert: tag name = 'TESTTAG' 
		description: 'A tagname was just set and could not be retrieved from the tag.'! !

ITTest subclass: #ITTestTranslator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:16'!
testDiscardTranslationsForLanguage
	"Tests whether all translations for a given language will be discarded."

	(ITTranslator getInstance loadTranslationFile: 'example.ts' forLanguage: #example).
	
	self
		assert: (ITTranslator getInstance translate: 'test' into: #example) = 'test'
		description: 'The test string should not be translated but was modified.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:14'!
testLoadTranslationFileForLanguage
	"Tests whether it is possible to load a translation file (a lookup dictionary or so)."
	
	(ITTranslator getInstance loadTranslationFile: 'example.ts' forLanguage: #example).
	
	self
		assert: (ITTranslator getInstance translate: 'test' into: #example) = 'test_translated'
		description: 'Could not translate test string.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 11:09'!
testSingleton
	"Tests whether ITTranslator is a singleton."
	
	self
		assert: ITTranslator getInstance == ITTranslator getInstance
		description: ITTranslator name, '(class)>>#getInstance returns variing objects. Should be a Singleton.'.
		
	[ITTranslator new]
		on: Exception
		do: [^self].
		
	self
		assert: false
		description: ITTranslator name, '(class)>>#new does not throws an Exception!! Should be a Singleton.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:00'!
testTr
	"Tests whether ByteString>>#tr does return a byte string as well."
	
	self
		assert: ('test' tr isKindOf: ByteString)
		description: 'ByteString>>#tr does not return an instance of ByteString.'! !

!ITTestTranslator methodsFor: 'testing' stamp: 'mt 12/26/2007 12:15'!
testTranslateInto
	"Tests whether the same text is returned if no translation is used."
	
	self
		assert: (ITTranslator getInstance translate: 'No Translation' into: #default) = 'No Translation'
		description: 'Text was modified while no translation was performed!!'! !

ITTest subclass: #ITTestUser
	instanceVariableNames: 'user adminRole guestRole'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:40'!
setUp

	super setUp.
	user := ITUser new.
	
	adminRole := ITRole new.
	adminRole rolename: 'admin'.
	
	guestRole := ITRole new.
	guestRole rolename: 'guest'.! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 10:59'!
testAddRole
	"Tests whether it is possible to add a role but not twice the same role."
	
	2 timesRepeat: [user addRole: guestRole].

	self
		assert: user roles size = 1
		description: 'A role was added twice to a user.'! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 10:36'!
testId
	"Tests whether the accessors for #id do what they should do - set and get."

	user id: 1.
	
	self
		assert: user id = 1 
		description: 'An id was just set and could not be retrieved from the user.'! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:00'!
testIsInRole
	"Tests whether it is possible to add a role to a user and ask this user whether he is in that role."
	
	user addRole: adminRole.
		
	self
		assert: (user isInRole: adminRole)
		description: 'A role was just added and could not be retrieved from a user.'! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 10:41'!
testPassword
	"Tests whether a password for a user can be set and retrieved as hash for security reasons."

	|password hash|
	password :=	'justsomestupidpasswordwhichwillneverbeinplaintextagain'.
	hash := ITSecurityManager hashPassword: password.
	
	user setPassword: password.
	
	self
		assert: user passwordHash = hash
		description: 'The user got another password hash as from ITSecurityManager(class)>>#hashPassword:.'
! !

!ITTestUser methodsFor: 'testing' stamp: 'mig 1/7/2008 12:15'!
testRemoveRole
	"Tests whether it is possible to remove a role."
	
	user
		addRole: adminRole;
		removeRole: adminRole.

	self
		assert: user roles isEmpty
		description: 'A role was added and could not be removed from a user'! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:22'!
testRoles
	"Tests whether the accessors for #roles do what they should do - set and get."

	| someRoles |
	someRoles := {adminRole. adminRole. guestRole. guestRole}.
	user roles: someRoles.
	
	self
		assert: (user roles isKindOf: Set) 
		description: 'The roles of an user are not a set.'.
		
	self
		assert: user roles = {adminRole. guestRole} asSet
		description: 'Some roles where just set to a user and could not be retrieved.' ! !

!ITTestUser methodsFor: 'testing' stamp: 'mt 12/30/2007 10:37'!
testUsername
	"Tests whether the accessors for #username do what they should do - set and get."

	user username: 'testuser'.
	
	self
		assert: user username = 'testuser' 
		description: 'A username was just set and could not be retrieved from the user.'! !

WAFileLibrary subclass: #ITFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITFileLibrary commentStamp: 'mt 12/30/2007 14:39' prior: 0!
This is the place for all (former) external resources. Style sheets, images, other files. You have access to these files with: "ITFileLibrary / #filenameExtension" which creates you a WAUrl instance for proper usage.

You should modify these files through the web frontend of seaside. But feel free to modify my methods as well.!

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
acceptPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 159 73 68 65 84 56 203 165 147 235 75 83 97 28 199 253 59 118 206 118 108 3 9 100 68 33 130 132 80 123 21 36 18 59 154 13 197 188 44 75 211 221 189 210 54 99 216 76 139 50 114 94 198 72 41 45 179 212 106 115 78 109 234 212 230 214 148 50 113 217 81 66 204 189 233 66 181 97 116 177 111 231 236 197 76 18 35 122 224 11 15 15 207 231 243 123 174 113 0 226 254 39 127 12 20 248 14 137 114 167 15 234 179 61 41 76 198 227 253 97 233 136 56 44 117 37 50 82 103 162 62 221 190 87 180 171 32 207 155 74 203 60 201 33 157 71 134 155 65 11 250 150 187 162 233 92 108 70 137 235 24 36 189 84 72 210 67 209 59 10 216 170 116 230 120 82 228 234 156 17 206 213 126 216 94 94 131 105 174 50 26 174 239 88 237 67 227 76 21 14 216 248 145 100 27 159 222 38 200 241 164 8 51 221 73 235 28 204 77 172 9 148 161 194 95 2 205 204 25 232 216 148 179 169 246 157 133 253 245 61 92 156 170 128 216 66 174 139 175 147 194 152 64 230 78 50 168 198 178 162 149 57 152 3 85 222 83 80 76 23 66 49 85 0 245 84 33 220 107 131 48 120 149 112 176 146 220 158 35 72 184 66 26 98 130 140 17 49 211 25 108 134 53 216 132 10 95 49 148 79 228 44 152 15 229 36 27 79 62 198 223 184 192 181 80 100 13 109 207 27 97 155 107 68 124 61 201 196 4 71 237 9 27 15 86 110 163 54 160 129 214 91 196 174 100 0 139 31 230 161 154 40 196 216 218 80 20 254 177 249 29 109 207 46 193 48 81 140 190 96 39 4 70 98 35 38 144 220 167 54 250 151 187 97 244 171 80 235 215 226 211 215 143 81 232 253 151 183 49 216 50 91 15 181 43 27 122 247 105 244 7 59 32 168 249 93 208 67 49 55 230 155 208 190 112 25 186 73 57 204 190 106 68 190 125 142 194 155 63 55 97 121 122 1 101 206 44 104 88 65 75 160 14 237 94 51 168 42 98 107 11 169 183 4 6 249 64 26 236 43 119 81 61 33 135 218 125 18 117 211 229 88 122 183 128 182 217 6 148 14 30 135 194 113 2 58 103 14 236 175 186 145 110 61 12 170 146 216 58 196 100 43 95 184 143 189 26 179 71 131 135 204 29 84 142 230 65 59 156 3 213 144 12 74 7 23 14 206 198 163 165 46 24 135 138 33 80 243 214 41 53 33 220 246 144 18 155 72 58 190 129 136 152 220 101 112 176 146 214 128 25 250 209 34 156 27 150 163 149 221 130 157 133 245 206 34 240 75 121 17 22 166 119 124 202 123 26 72 154 50 17 33 105 135 4 237 126 51 122 95 88 209 59 111 133 197 107 66 90 75 42 4 10 94 136 82 17 244 174 159 137 58 79 138 40 3 161 167 106 8 70 160 229 133 5 42 94 152 173 200 176 209 83 165 132 232 175 191 241 95 243 11 103 208 172 229 121 186 212 99 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
addPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 111 73 68 65 84 56 203 165 147 235 75 147 97 24 135 253 91 182 47 5 131 164 134 89 40 168 41 37 88 40 111 217 108 138 138 78 219 150 115 107 138 206 246 110 46 157 186 205 3 45 157 19 15 21 181 161 219 104 164 59 56 15 195 3 102 226 212 210 69 234 235 80 162 141 34 8 106 195 175 191 222 21 77 71 203 136 30 248 125 121 224 186 158 231 190 185 239 36 0 73 255 147 223 46 116 211 117 204 14 143 132 108 117 137 168 230 9 65 88 254 162 58 220 240 188 130 146 90 203 72 209 104 49 243 68 129 110 90 74 180 79 74 66 207 214 123 177 26 90 132 255 243 6 214 63 173 96 50 96 135 210 83 3 190 137 19 226 61 185 78 36 20 252 132 197 145 233 61 59 14 190 237 97 246 131 11 38 106 16 143 119 12 176 239 155 176 244 113 14 218 5 25 74 71 242 35 220 193 60 34 78 160 157 146 50 104 56 232 222 181 96 239 235 54 134 183 245 120 184 214 129 110 95 43 12 126 29 250 252 90 116 111 182 192 125 96 7 233 17 162 208 120 37 88 208 155 205 136 9 104 88 209 191 172 198 187 47 155 244 139 125 232 221 210 66 243 74 137 95 71 189 38 131 124 81 8 197 114 45 156 251 54 220 28 201 65 222 131 12 69 76 208 226 172 161 92 1 27 253 85 51 58 87 85 104 91 145 67 54 43 138 9 4 54 46 26 102 202 32 157 42 198 192 27 29 244 75 205 184 220 157 70 197 4 138 113 254 161 247 189 11 253 111 117 52 220 132 63 29 241 100 17 20 243 124 88 252 195 200 210 165 30 198 4 77 118 222 225 68 96 12 218 13 18 42 95 253 137 2 210 91 13 171 127 8 233 29 236 35 65 189 173 156 50 250 180 48 108 105 208 244 82 8 241 124 5 120 214 27 113 96 52 119 61 92 244 251 212 232 242 222 195 5 117 242 81 9 226 177 18 197 109 43 1 71 192 2 217 124 37 36 222 210 31 53 31 23 136 156 28 212 185 139 225 216 53 227 82 79 42 216 247 89 71 77 20 154 139 24 85 79 11 131 164 71 128 113 106 52 214 176 40 88 235 140 38 10 115 49 177 99 194 29 203 173 40 28 76 86 158 102 196 13 82 249 163 2 162 100 232 106 164 209 81 9 39 45 49 250 218 65 206 84 65 62 85 9 227 106 27 28 52 44 180 112 192 86 177 34 52 76 36 28 101 206 64 46 65 24 114 66 249 3 153 232 89 32 97 126 109 128 121 195 0 141 183 17 89 93 41 81 56 116 28 78 184 76 215 244 153 204 220 158 116 50 187 243 34 149 161 73 9 167 181 159 13 159 111 61 67 157 83 177 200 100 229 41 230 95 183 241 95 243 29 65 70 203 31 0 40 211 193 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
binPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 4 0 0 0 181 250 55 234 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 110 73 68 65 84 24 25 5 193 191 75 84 113 0 0 240 207 123 94 202 157 215 21 69 22 53 84 168 68 100 17 221 210 208 32 68 129 109 70 109 246 7 164 155 213 214 218 84 91 52 59 244 131 22 201 90 130 104 9 196 254 0 27 36 78 7 3 15 2 233 56 61 125 119 239 251 222 183 207 39 137 0 0 214 158 141 76 116 222 222 94 5 146 8 128 167 183 226 189 201 7 179 147 85 91 217 187 55 253 149 215 171 136 162 40 138 158 204 45 182 190 197 141 184 31 139 248 55 126 143 11 173 249 185 40 5 32 157 191 63 62 233 164 125 125 195 174 154 30 79 231 169 0 159 147 194 212 212 21 137 3 35 50 163 118 53 109 93 167 194 151 161 242 124 113 182 24 62 86 111 153 80 85 74 28 56 225 43 85 42 148 151 138 231 97 58 212 179 164 106 195 161 68 48 34 115 193 22 42 159 234 197 171 169 153 211 142 232 88 183 109 92 162 16 4 29 3 164 225 238 216 204 25 135 50 13 55 29 181 105 84 162 166 45 202 145 230 143 47 218 181 167 107 95 162 105 83 148 26 50 16 229 72 243 83 21 127 100 50 61 7 234 186 254 129 129 66 142 116 176 221 115 89 219 47 93 61 244 245 149 74 185 82 64 58 248 248 91 221 53 55 52 144 43 28 87 138 134 212 228 72 23 62 172 47 255 208 208 80 85 179 102 194 152 115 26 238 104 11 59 84 24 60 252 185 220 157 109 42 229 86 98 61 121 41 8 138 188 216 41 154 36 17 44 62 202 95 132 118 216 11 239 151 150 0 128 255 200 133 159 94 23 25 132 161 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
cancelPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 221 73 68 65 84 56 203 165 147 107 107 26 65 20 134 247 107 72 32 33 41 166 52 132 150 94 99 155 160 168 161 198 152 20 52 101 141 74 155 218 75 164 105 40 189 209 203 255 204 135 92 32 127 163 196 149 160 178 206 174 187 59 79 103 86 55 85 148 150 210 15 47 11 195 190 207 153 247 156 51 6 96 252 143 198 15 13 195 10 143 7 10 70 101 253 25 96 24 13 30 221 135 239 239 224 219 1 242 235 91 228 151 58 242 211 27 228 135 87 248 15 110 227 25 70 123 50 32 50 255 56 4 179 136 204 100 144 169 20 65 50 73 176 182 134 159 207 226 31 62 167 119 239 38 66 255 59 2 24 54 31 212 225 252 28 89 171 17 36 18 4 171 171 248 149 10 242 236 12 175 90 194 219 223 197 189 179 140 61 128 68 128 254 149 85 101 109 166 211 129 102 19 191 90 197 47 151 193 82 209 219 109 228 201 9 189 196 67 122 47 10 116 66 235 16 32 204 155 78 19 236 237 133 230 8 18 74 155 21 164 183 189 141 179 184 136 251 236 9 173 97 128 238 176 252 188 255 59 175 186 114 100 12 165 204 238 214 22 78 44 134 88 88 192 169 228 39 0 62 190 238 231 141 199 241 75 165 17 128 174 238 230 114 136 249 121 186 179 179 56 165 236 56 32 120 95 195 95 89 193 55 205 171 204 250 171 205 180 90 200 139 11 132 154 140 61 61 141 243 116 157 203 97 128 167 1 106 68 222 227 20 242 244 180 95 181 209 192 221 220 196 217 216 8 205 82 65 130 163 35 186 203 215 17 197 244 40 192 85 27 166 231 235 213 203 120 102 129 224 248 56 52 235 188 221 185 57 132 110 174 50 139 245 36 98 39 67 123 233 26 77 195 248 57 178 72 106 57 44 61 223 222 203 29 220 248 221 171 188 246 204 12 246 212 20 246 82 44 172 220 26 50 143 173 178 90 14 203 190 117 3 183 154 199 217 205 225 152 217 48 175 40 102 16 133 212 152 121 226 99 82 221 181 116 135 35 93 14 212 236 171 249 247 215 248 143 250 5 231 149 184 131 253 158 221 191 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
commentsPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 1 191 73 68 65 84 56 203 141 146 189 79 194 80 20 197 89 77 76 156 220 140 137 131 137 131 139 206 254 27 38 46 234 160 127 129 198 93 23 25 36 36 206 186 168 147 6 72 212 65 196 60 226 128 147 17 67 88 208 224 71 16 249 40 21 40 31 133 182 226 245 220 71 73 196 208 234 240 75 251 94 239 57 239 222 211 231 89 240 134 167 128 31 156 58 112 12 86 192 56 17 121 126 227 193 135 221 104 34 43 106 186 41 180 30 205 46 85 80 172 234 34 24 75 11 212 109 58 25 156 177 184 130 226 114 195 20 31 117 83 148 234 134 40 213 12 81 212 218 114 63 163 212 217 224 208 209 128 79 101 177 98 139 10 213 182 200 85 90 34 91 110 201 245 95 6 231 181 150 41 84 156 204 197 121 136 175 147 185 219 245 189 88 121 201 119 101 45 3 126 162 206 0 21 155 44 216 230 252 216 32 160 66 104 88 29 41 230 147 55 246 111 170 200 133 48 26 105 61 154 93 144 11 33 23 66 46 4 109 144 13 86 125 129 184 136 167 21 97 126 118 36 139 59 17 139 197 152 159 48 26 33 23 66 46 132 92 8 93 202 125 140 197 6 143 114 14 188 204 129 19 78 252 242 46 195 243 90 124 42 139 21 91 132 92 8 221 17 114 145 235 62 3 219 228 32 18 151 226 48 168 35 23 66 46 178 56 255 67 156 81 117 202 227 253 93 109 176 193 75 79 60 196 97 130 11 48 9 10 200 133 218 86 167 79 204 93 40 216 71 94 244 90 172 177 65 170 103 48 10 182 192 132 189 246 250 67 247 148 120 46 17 50 145 176 217 91 169 65 79 121 141 142 162 41 22 235 96 205 51 232 223 218 38 243 64 211 116 131 144 11 11 190 64 26 36 249 246 130 17 121 15 156 12 108 147 7 228 194 226 54 152 25 120 145 92 196 195 220 1 104 129 89 167 58 55 131 49 16 2 211 110 93 186 142 240 31 190 1 193 159 9 148 122 211 60 183 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
datePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 4 73 68 65 84 56 203 149 82 109 107 26 65 16 126 238 188 212 139 231 75 36 20 26 130 136 53 88 164 37 162 32 8 237 199 130 33 224 143 243 179 191 35 144 63 80 138 133 64 80 17 141 90 40 42 69 250 65 197 35 190 220 185 219 153 141 119 152 212 22 58 240 48 187 59 51 207 188 236 64 74 9 6 137 150 205 102 63 87 171 213 111 153 76 230 138 238 6 191 237 96 20 10 133 79 181 90 237 43 251 240 155 23 167 14 3 77 147 191 216 235 230 6 231 249 60 134 119 119 16 149 10 94 227 73 216 22 188 189 197 155 203 75 252 188 191 199 230 250 90 217 210 82 106 186 70 194 78 86 48 8 43 22 195 118 187 69 56 30 199 113 32 128 184 174 43 152 228 98 70 34 202 118 28 141 194 220 17 115 172 190 59 195 52 12 116 6 3 229 212 237 245 32 136 196 62 57 129 29 14 99 107 154 232 116 187 202 214 121 120 240 9 224 245 216 7 68 140 178 55 206 206 208 179 109 92 80 246 20 181 166 83 128 116 28 172 93 23 223 73 15 8 111 133 64 129 244 35 217 47 0 221 216 175 224 195 233 41 222 91 22 228 102 3 151 156 4 131 200 94 17 209 187 163 35 100 72 107 132 224 98 129 71 34 245 171 168 215 235 210 117 221 191 194 113 28 185 92 46 21 88 248 141 99 56 214 192 1 89 175 215 104 183 219 72 167 211 232 247 251 170 247 249 124 142 114 185 236 125 185 47 250 33 2 14 30 141 70 136 208 228 115 185 28 82 169 20 66 161 16 40 179 15 143 232 25 129 247 152 167 93 240 22 133 209 106 181 84 53 14 15 148 170 163 86 252 152 63 90 120 89 226 108 54 131 160 201 115 240 100 50 81 103 134 231 119 112 6 251 100 220 14 173 182 154 67 50 153 228 229 193 112 56 132 65 191 198 3 213 229 142 138 148 216 15 174 208 42 179 169 88 44 194 162 175 229 172 28 60 30 143 125 34 127 6 11 250 87 155 132 142 130 13 251 8 208 30 176 230 10 56 115 34 145 192 106 181 82 45 249 45 116 105 77 201 49 202 101 253 75 188 10 152 172 217 108 162 84 42 61 17 76 167 211 47 141 70 227 35 254 67 232 55 126 176 254 13 45 58 77 7 87 49 185 249 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
deletePng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 16 0 0 0 16 8 6 0 0 0 31 243 255 97 0 0 0 4 103 65 77 65 0 0 175 200 55 5 138 233 0 0 0 25 116 69 88 116 83 111 102 116 119 97 114 101 0 65 100 111 98 101 32 73 109 97 103 101 82 101 97 100 121 113 201 101 60 0 0 2 93 73 68 65 84 56 203 165 147 251 75 83 97 24 199 253 91 182 31 162 4 137 110 132 132 81 80 152 243 50 119 214 218 220 166 206 179 76 143 91 44 98 105 142 157 97 154 65 141 197 92 248 67 168 165 118 213 202 95 50 77 108 148 90 70 106 215 161 229 177 242 210 78 77 106 109 158 179 107 202 183 185 96 38 46 35 122 225 251 203 203 251 249 188 60 15 207 147 2 32 229 127 178 230 98 86 175 23 206 80 21 230 247 84 25 51 165 37 185 73 173 134 123 71 170 153 113 82 105 118 149 200 133 235 10 230 116 122 226 35 69 177 223 28 54 132 7 157 136 188 25 69 100 100 8 252 221 14 204 26 74 241 170 144 96 159 171 197 68 82 193 50 60 93 78 241 11 183 59 176 52 245 22 209 190 59 136 182 219 17 109 62 135 31 55 155 176 56 220 7 143 201 128 81 101 54 255 76 158 73 172 18 204 232 116 130 24 236 230 174 183 99 137 113 33 122 241 12 124 118 11 252 182 106 132 47 88 16 105 160 17 182 158 64 248 222 13 204 29 37 49 124 104 159 251 177 108 143 32 33 136 193 244 124 173 25 139 174 177 248 143 33 7 13 239 89 35 130 117 186 85 225 78 146 8 119 93 193 203 188 12 12 72 51 232 132 224 3 117 132 9 116 93 69 180 179 25 62 107 37 190 22 73 147 102 161 146 4 111 171 129 199 82 133 135 68 58 147 16 48 229 218 96 228 126 23 162 14 11 124 167 13 248 211 241 40 114 224 165 10 225 111 110 132 51 111 71 48 33 152 36 139 130 161 206 86 132 235 13 8 158 42 91 87 48 95 170 130 191 169 17 253 226 45 43 130 137 18 21 227 181 214 32 100 167 193 29 87 199 31 38 141 50 15 190 90 19 102 77 70 244 137 210 86 74 112 21 203 105 70 38 66 176 179 13 62 173 12 222 82 201 26 152 149 103 131 45 144 32 208 126 9 67 226 109 232 205 218 180 210 196 215 69 82 193 11 13 225 158 171 208 32 112 171 53 222 176 121 149 248 23 168 200 5 43 139 193 50 49 248 182 22 140 23 75 151 97 119 183 104 163 96 213 32 141 21 228 16 35 138 3 252 244 97 21 2 215 90 241 189 158 134 135 84 226 179 90 1 111 157 25 252 229 22 76 168 243 209 147 149 202 199 96 34 233 40 63 149 239 39 158 28 220 203 142 74 118 225 75 181 17 222 134 243 241 124 170 58 134 71 57 91 151 97 246 119 56 233 50 13 74 119 11 7 196 233 102 39 177 147 121 144 191 157 235 23 109 230 122 115 211 152 158 236 84 115 119 230 6 225 95 183 241 95 243 19 29 210 206 185 73 114 27 254 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
hpidesignCss
	^ '/* general settings for all components */
body {
	margin: 0px;
	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
}

:link, :visited {
	color: #007a9e;
	text-decoration: none;
	border-bottom: 1px dotted #aeb3b4;
}

:link:hover, :visited:hover {
	background-color: #dfe1e1;
	color: #4ca2bb;
}

.content {
	border:1px #000000 dotted;	/* ONLY DEBUG */
	min-width: 700px;
	max-width: 100%;
	margin: 5px;
}

fieldset {
	margin: 5px;
	color: #aeb3b4;
	border: 1px solid #aeb3b4;
	font-weight: bold;
}

fieldset ul li {
	color: #000;
	font-weight: normal;
}

/* links/anchors with images */
:link img, :visited img{
	color: #fff;
	background-color: #fff
	
	text-decoration: none;
	border: none;
	
	display: block;
}

:link:hover img, :visited:hover img{
	color: #09c;
	background-color: #09c
}

/* a div button */
.button {
	margin: 10px auto;
	background-color: #dfe1e1;
	border: 1px dotted #aeb3b4;
	width: 100px;
	padding: 5px;
	text-align: center;
}

/* top-left (project name) and login menu */
#project_name {
	float:left;
	margin: 0px;
	padding: 5px;
}

#login_menu {
	text-align: right;
	color: #aeb3b4;
	margin: 5px;
	padding: 5px;
	float:right;
}

#login_menu .text {
	color: #5e676a;
}

/* main menu */
#main_menu {
	clear: left;
	margin: .66em 5px;
	padding: .2em 0;
	
	border: none;
	border-right: 1px solid #000;
	
	background-color: #dfe1e1;
}

#main_menu ul {
	margin: 0;
	font-size: 10px;
	text-align: right;
	list-style: none;
}

#main_menu li {
	padding: .25em 0;
	display: inline;
	white-space: nowrap;
}

#main_menu :link, #main_menu :visited {
	color: #000;
	background-color: #fff;
	
	border: 1px solid #000;
	border-right: none;
	
	padding: .2em 20px;
}

#main_menu :link:hover, #main_menu :visited:hover {
	background-color: #aeb3b4;
}

#main_menu .active :link, #main_menu .active :visited {
	font-weight: bold;
	color: #dfe1e1;
	background-color: #000;
}

/* Footer */

.footer {
	border-top: 1px #007a9e dashed;
	margin-top: 25px;
}

.footer_right {
	padding: 0.5em 1em;
	text-align: right;
	color: #9CA0AD;
	float:right;
}


/* side menu (left) */
#tab_menu {
	border: 1px solid black;
	border-color: #aeb3b4 #5e676a #5e676a #aeb3b4;
	margin: 2em 5px;
	padding: .5em 0 0;
	float: left;
	width: 12em;
}

#tab_menu ul { 
	list-style: none;
	margin: 0 0 .5em;
	padding: 0;
}

#tab_menu li {
	margin: 0;
	padding: 0.1em .5em;
	font-weight: bold;
}
#tab_menu li li {
	margin: 0 -0.5em;
	padding-left: 1.5em;
	font-weight: normal
}

#tab_menu li li.active {
	background: #cce4ec;
}
#tab_menu :link, #tab_menu :visited {
	border: none;
	display: block
}
#tab_menu :link:hover, #tab_menu :visited:hover {
	background: transparent
}

.content_with_menu {
	padding: 0.4em 2em;
	margin-left: 12em;
	min-height: 300px;
	width: 400px;
	max-width: 100%
}

/* detailed view */
#issue {
 background: #dfe1e1;
 border: 1px solid #5e676a;
 margin-top: 1em;
 padding: .5em 1em;
 position: relative;
}

#issue h2 {
	margin: 0 0 .8em 0;
}

#issue table.details {
 border-top: 1px solid #5e676a;
 border-collapse: collapse;
 table-layout: fixed;
 width: 100%;
}

#issue table.details tr { 
	border-bottom: 1px dotted #8e9496;
}

#issue table.details td, #issue table.details th {
 font-size: 80%;
 padding: .5em 1em;
 vertical-align: top;
}

#issue table.details th {
 color: #5e676a;
 font-weight: bold;
 text-align: left;
 width: 20%;
}

#issue table.details td {
	width: 30%;
}

#issue table.details .description {
	border-top: 1px solid #5e676a;
}

#issue .description h3 {
 border-bottom: 1px solid #5e676a;
 color: #5e676a;
 font-size: 100%;
 font-weight: bold;
}

.comment {
	background: #cce4ec;
	padding: 0.2em 1em 1em 1em;
	border: 1px dashed #007a9e;
	margin-bottom: 1em;
}

.comment h3 {
 border-bottom: 1px solid #007a9e;
 color: #007a9e;
 font-size: 100%;
 font-weight: normal;
}

/* drag & droppable lists */

fieldset.drag_list {
	float: left;
	width: 170px;
}

fieldset.drag_list ul {
	min-height: 50px;
	color: #000;
	
	list-style: none;
	padding: 0;
	margin: 0;
}

fieldset.drag_list ul li {
	margin: 4px;
	padding: 2px;
	cursor: move;
	font-weight: normal;
	text-align: center;
}

fieldset.drag_list ul li.available {
	background-color: #dfe1e1;
  	border: 1px dashed #5e676a;
}
fieldset.drag_list ul li.attached {
	background-color: #cce4ec;
  	border: 1px solid #007a9e;
}

/* list issues table */

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block
}

.list_issues {
	border:1px #000000 dotted;	/* ONLY DEBUG */
	border-spacing: 0px;
	table-layout: automatic;
	width:100%;
}

.list_issues thead th {
	
	background-color: #fff;
	text-transform: capitalize;
	vertical-align: middle;
	
	border-left: 1px dotted #8e9496;
	border-bottom: 1px solid #8e9496;
	
	padding: .3em 1em;
}

.list_issues tbody tr {
	text-align: center;
}

.list_issues tbody td {
	padding: .1em .5em;
	border-bottom: 1px solid #8e9496;
	border-left: 1px solid #8e9496;
}

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block;
	font-weight: bold;
	border: none;
	background: none;
	color: #000;
}

.list_issues tbody tr:hover {
	background: #7fbcce;
}

.list_issues tr.odd {
	background-color: #dfe1e1;
}

.list_issues tr.even {
	background-color: #fff;
}

/* message box */
div.messagebox {
	background-color: white;
	border: 1px solid black;
	color: black;
	padding: 1em;
}

div.messagebox .title {	
	border-bottom: 1px dotted #8e9496;
	font-weight: bold;
	margin-bottom: 1ex;
	padding-bottom: 0.3ex;
}

div.messagebox .buttons {
	margin-top: 2ex;
	text-align: center;
}

div.messagebox .buttons input {
	margin-left: 5px;
	margin-right: 5px;
	min-width: 5em;
	border: 1px solid #8e9496;
	background-color: #dfe1e1;
}'! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
logobottomPng
	| cache |
	cache := #(nil) beMutable.
	(cache at: 1) isNil ifTrue: [
		cache at: 1 put: #(137 80 78 71 13 10 26 10 0 0 0 13 73 72 68 82 0 0 0 111 0 0 0 69 8 2 0 0 0 203 144 47 103 0 0 0 9 112 72 89 115 0 0 11 19 0 0 11 19 1 0 154 156 24 0 0 10 79 105 67 67 80 80 104 111 116 111 115 104 111 112 32 73 67 67 32 112 114 111 102 105 108 101 0 0 120 218 157 83 103 84 83 233 22 61 247 222 244 66 75 136 128 148 75 111 82 21 8 32 82 66 139 128 20 145 38 42 33 9 16 74 136 33 161 217 21 81 193 17 69 69 4 27 200 160 136 3 142 142 128 140 21 81 44 12 138 10 216 7 228 33 162 142 131 163 136 138 202 251 225 123 163 107 214 188 247 230 205 254 181 215 62 231 172 243 157 179 207 7 192 8 12 150 72 51 81 53 128 12 169 66 30 17 224 131 199 196 198 225 228 46 64 129 10 36 112 0 16 8 179 100 33 115 253 35 1 0 248 126 60 60 43 34 192 7 190 0 1 120 211 11 8 0 192 77 155 192 48 28 135 255 15 234 66 153 92 1 128 132 1 192 116 145 56 75 8 128 20 0 64 122 142 66 166 0 64 70 1 128 157 152 38 83 0 160 4 0 96 203 99 98 227 0 80 45 0 96 39 127 230 211 0 128 157 248 153 123 1 0 91 148 33 21 1 160 145 0 32 19 101 136 68 0 104 59 0 172 207 86 138 69 0 88 48 0 20 102 75 196 57 0 216 45 0 48 73 87 102 72 0 176 183 0 192 206 16 11 178 0 8 12 0 48 81 136 133 41 0 4 123 0 96 200 35 35 120 0 132 153 0 20 70 242 87 60 241 43 174 16 231 42 0 0 120 153 178 60 185 36 57 69 129 91 8 45 113 7 87 87 46 30 40 206 73 23 43 20 54 97 2 97 154 64 46 194 121 153 25 50 129 52 15 224 243 204 0 0 160 145 21 17 224 131 243 253 120 206 14 174 206 206 54 142 182 14 95 45 234 191 6 255 34 98 98 227 254 229 207 171 112 64 0 0 225 116 126 209 254 44 47 179 26 128 59 6 128 109 254 162 37 238 4 104 94 11 160 117 247 139 102 178 15 64 181 0 160 233 218 87 243 112 248 126 60 60 69 161 144 185 217 217 229 228 228 216 74 196 66 91 97 202 87 125 254 103 194 95 192 87 253 108 249 126 60 252 247 245 224 190 226 36 129 50 93 129 71 4 248 224 194 204 244 76 165 28 207 146 9 132 98 220 230 143 71 252 183 11 255 252 29 211 34 196 73 98 185 88 42 20 227 81 18 113 142 68 154 140 243 50 165 34 137 66 146 41 197 37 210 255 100 226 223 44 251 3 62 223 53 0 176 106 62 1 123 145 45 168 93 99 3 246 75 39 16 88 116 192 226 247 0 0 242 187 111 193 212 40 8 3 128 104 131 225 207 119 255 239 63 253 71 160 37 0 128 102 73 146 113 0 0 94 68 36 46 84 202 179 63 199 8 0 0 68 160 129 42 176 65 27 244 193 24 44 192 6 28 193 5 220 193 11 252 96 54 132 66 36 196 194 66 16 66 10 100 128 28 114 96 41 172 130 66 40 134 205 176 29 42 96 47 212 64 29 52 192 81 104 134 147 112 14 46 194 85 184 14 61 112 15 250 97 8 158 193 40 188 129 9 4 65 200 8 19 97 33 218 136 1 98 138 88 35 142 8 23 153 133 248 33 193 72 4 18 139 36 32 201 136 20 81 34 75 145 53 72 49 82 138 84 32 85 72 29 242 61 114 2 57 135 92 70 186 145 59 200 0 50 130 252 134 188 71 49 148 129 178 81 61 212 12 181 67 185 168 55 26 132 70 162 11 208 100 116 49 154 143 22 160 155 208 114 180 26 61 140 54 161 231 208 171 104 15 218 143 62 67 199 48 192 232 24 7 51 196 108 48 46 198 195 66 177 56 44 9 147 99 203 177 34 172 12 171 198 26 176 86 172 3 187 137 245 99 207 177 119 4 18 129 69 192 9 54 4 119 66 32 97 30 65 72 88 76 88 78 216 72 168 32 28 36 52 17 218 9 55 9 3 132 81 194 39 34 147 168 75 180 38 186 17 249 196 24 98 50 49 135 88 72 44 35 214 18 143 19 47 16 123 136 67 196 55 36 18 137 67 50 39 185 144 2 73 177 164 84 210 18 210 70 210 110 82 35 233 44 169 155 52 72 26 35 147 201 218 100 107 178 7 57 148 44 32 43 200 133 228 157 228 195 228 51 228 27 228 33 242 91 10 157 98 64 113 164 248 83 226 40 82 202 106 74 25 229 16 229 52 229 6 101 152 50 65 85 163 154 82 221 168 161 84 17 53 143 90 66 173 161 182 82 175 81 135 168 19 52 117 154 57 205 131 22 73 75 165 173 162 149 211 26 104 23 104 247 105 175 232 116 186 17 221 149 30 78 151 208 87 210 203 233 71 232 151 232 3 244 119 12 13 134 21 131 199 136 103 40 25 155 24 7 24 103 25 119 24 175 152 76 166 25 211 139 25 199 84 48 55 49 235 152 231 153 15 153 111 85 88 42 182 42 124 21 145 202 10 149 74 149 38 149 27 42 47 84 169 170 166 170 222 170 11 85 243 85 203 84 143 169 94 83 125 174 70 85 51 83 227 169 9 212 150 171 85 170 157 80 235 83 27 83 103 169 59 168 135 170 103 168 111 84 63 164 126 89 253 137 6 89 195 76 195 79 67 164 81 160 177 95 227 188 198 32 11 99 25 179 120 44 33 107 13 171 134 117 129 53 196 38 177 205 217 124 118 42 187 152 253 29 187 139 61 170 169 161 57 67 51 74 51 87 179 82 243 148 102 63 7 227 152 113 248 156 116 78 9 231 40 167 151 243 126 138 222 20 239 41 226 41 27 166 52 76 185 49 101 92 107 170 150 151 150 88 171 72 171 81 171 71 235 189 54 174 237 167 157 166 189 69 187 89 251 129 14 65 199 74 39 92 39 71 103 143 206 5 157 231 83 217 83 221 167 10 167 22 77 61 58 245 174 46 170 107 165 27 161 187 68 119 191 110 167 238 152 158 190 94 128 158 76 111 167 222 121 189 231 250 28 125 47 253 84 253 109 250 167 245 71 12 88 6 179 12 36 6 219 12 206 24 60 197 53 113 111 60 29 47 199 219 241 81 67 93 195 64 67 165 97 149 97 151 225 132 145 185 209 60 163 213 70 141 70 15 140 105 198 92 227 36 227 109 198 109 198 163 38 6 38 33 38 75 77 234 77 238 154 82 77 185 166 41 166 59 76 59 76 199 205 204 205 162 205 214 153 53 155 61 49 215 50 231 155 231 155 215 155 223 183 96 90 120 90 44 182 168 182 184 101 73 178 228 90 166 89 238 182 188 110 133 90 57 89 165 88 85 90 93 179 70 173 157 173 37 214 187 173 187 167 17 167 185 78 147 78 171 158 214 103 195 176 241 182 201 182 169 183 25 176 229 216 6 219 174 182 109 182 125 97 103 98 23 103 183 197 174 195 238 147 189 147 125 186 125 141 253 61 7 13 135 217 14 171 29 90 29 126 115 180 114 20 58 86 58 222 154 206 156 238 63 125 197 244 150 233 47 103 88 207 16 207 216 51 227 182 19 203 41 196 105 157 83 155 211 71 103 23 103 185 115 131 243 136 139 137 75 130 203 46 151 62 46 155 27 198 221 200 189 228 74 116 245 113 93 225 122 210 245 157 155 179 155 194 237 168 219 175 238 54 238 105 238 135 220 159 204 52 159 41 158 89 51 115 208 195 200 67 224 81 229 209 63 11 159 149 48 107 223 172 126 79 67 79 129 103 181 231 35 47 99 47 145 87 173 215 176 183 165 119 170 247 97 239 23 62 246 62 114 159 227 62 227 60 55 222 50 222 89 95 204 55 192 183 200 183 203 79 195 111 158 95 133 223 67 127 35 255 100 255 122 255 209 0 167 128 37 1 103 3 137 129 65 129 91 2 251 248 122 124 33 191 142 63 58 219 101 246 178 217 237 65 140 160 185 65 21 65 143 130 173 130 229 193 173 33 104 200 236 144 173 33 247 231 152 206 145 206 105 14 133 80 126 232 214 208 7 97 230 97 139 195 126 12 39 133 135 133 87 134 63 142 112 136 88 26 209 49 151 53 119 209 220 67 115 223 68 250 68 150 68 222 155 103 49 79 57 175 45 74 53 42 62 170 46 106 60 218 55 186 52 186 63 198 46 102 89 204 213 88 157 88 73 108 75 28 57 46 42 174 54 110 108 190 223 252 237 243 135 226 157 226 11 227 123 23 152 47 200 93 112 121 161 206 194 244 133 167 22 169 46 18 44 58 150 64 76 136 78 56 148 240 65 16 42 168 22 140 37 242 19 119 37 142 10 121 194 29 194 103 34 47 209 54 209 136 216 67 92 42 30 78 242 72 42 77 122 146 236 145 188 53 121 36 197 51 165 44 229 185 132 39 169 144 188 76 13 76 221 155 58 158 22 154 118 32 109 50 61 58 189 49 131 146 145 144 113 66 170 33 77 147 182 103 234 103 230 102 118 203 172 101 133 178 254 197 110 139 183 47 30 149 7 201 107 179 144 172 5 89 45 10 182 66 166 232 84 90 40 215 42 7 178 103 101 87 102 191 205 137 202 57 150 171 158 43 205 237 204 179 202 219 144 55 156 239 159 255 237 18 194 18 225 146 182 165 134 75 87 45 29 88 230 189 172 106 57 178 60 113 121 219 10 227 21 5 43 134 86 6 172 60 184 138 182 42 109 213 79 171 237 87 151 174 126 189 38 122 77 107 129 94 193 202 130 193 181 1 107 235 11 85 10 229 133 125 235 220 215 237 93 79 88 47 89 223 181 97 250 134 157 27 62 21 137 138 174 20 219 23 151 21 127 216 40 220 120 229 27 135 111 202 191 153 220 148 180 169 171 196 185 100 207 102 210 102 233 230 222 45 158 91 14 150 170 151 230 151 14 110 13 217 218 180 13 223 86 180 237 245 246 69 219 47 151 205 40 219 187 131 182 67 185 163 191 60 184 188 101 167 201 206 205 59 63 84 164 84 244 84 250 84 54 238 210 221 181 97 215 248 110 209 238 27 123 188 246 52 236 213 219 91 188 247 253 62 201 190 219 85 1 85 77 213 102 213 101 251 73 251 179 247 63 174 137 170 233 248 150 251 109 93 173 78 109 113 237 199 3 210 3 253 7 35 14 182 215 185 212 213 29 210 61 84 82 143 214 43 235 71 14 199 31 190 254 157 239 119 45 13 54 13 85 141 156 198 226 35 112 68 121 228 233 247 9 223 247 30 13 58 218 118 140 123 172 225 7 211 31 118 29 103 29 47 106 66 154 242 154 70 155 83 154 251 91 98 91 186 79 204 62 209 214 234 222 122 252 71 219 31 15 156 52 60 89 121 74 243 84 201 105 218 233 130 211 147 103 242 207 140 157 149 157 125 126 46 249 220 96 219 162 182 123 231 99 206 223 106 15 111 239 186 16 116 225 210 69 255 139 231 59 188 59 206 92 242 184 116 242 178 219 229 19 87 184 87 154 175 58 95 109 234 116 234 60 254 147 211 79 199 187 156 187 154 174 185 92 107 185 238 122 189 181 123 102 247 233 27 158 55 206 221 244 189 121 241 22 255 214 213 158 57 61 221 189 243 122 111 247 197 247 245 223 22 221 126 114 39 253 206 203 187 217 119 39 238 173 188 79 188 95 244 64 237 65 217 67 221 135 213 63 91 254 220 216 239 220 127 106 192 119 160 243 209 220 71 247 6 133 131 207 254 145 245 143 15 67 5 143 153 143 203 134 13 134 235 158 56 62 57 57 226 63 114 253 233 252 167 67 207 100 207 38 158 23 254 162 254 203 174 23 22 47 126 248 213 235 215 206 209 152 209 161 151 242 151 147 191 109 124 165 253 234 192 235 25 175 219 198 194 198 30 190 201 120 51 49 94 244 86 251 237 193 119 220 119 29 239 163 223 15 79 228 124 32 127 40 255 104 249 177 245 83 208 167 251 147 25 147 147 255 4 3 152 243 252 99 51 45 219 0 0 0 32 99 72 82 77 0 0 122 37 0 0 128 131 0 0 249 255 0 0 128 233 0 0 117 48 0 0 234 96 0 0 58 152 0 0 23 111 146 95 197 70 0 0 22 160 73 68 65 84 120 218 236 92 121 92 85 213 246 95 251 156 59 112 239 5 46 51 92 153 68 48 21 145 73 52 205 137 44 45 223 79 211 39 168 253 34 43 181 167 57 61 109 146 210 94 22 63 181 87 124 196 71 54 104 243 83 95 35 106 102 250 82 139 16 120 226 128 134 36 56 32 10 200 36 16 112 129 59 157 115 246 222 191 63 182 29 111 112 25 84 234 245 73 214 95 155 189 215 222 251 156 239 89 107 175 97 175 11 162 148 66 31 245 18 161 62 52 251 208 236 67 179 15 205 62 234 53 52 45 34 105 182 138 205 86 177 197 134 205 2 17 41 149 48 33 64 57 10 0 192 115 72 173 224 156 85 10 157 154 119 86 242 206 42 94 167 228 85 10 174 15 205 246 100 149 72 201 79 150 234 22 91 155 77 20 49 38 8 36 12 24 0 19 66 40 5 10 18 166 148 82 129 80 145 82 76 168 2 144 134 231 116 106 228 169 83 246 115 214 244 211 59 25 92 157 208 111 251 38 229 229 229 38 147 41 60 60 252 247 133 166 69 196 121 149 173 245 109 2 135 128 2 161 148 98 74 37 130 8 0 33 132 18 130 41 16 2 24 19 27 33 2 33 0 64 48 21 68 106 195 34 161 132 71 8 113 200 224 170 29 228 169 9 245 212 5 186 105 149 252 175 46 176 159 125 246 217 178 101 203 76 38 211 202 149 43 215 173 91 199 113 220 239 5 205 226 122 115 65 109 171 2 1 5 74 8 197 64 9 65 152 80 12 64 9 166 4 99 130 68 137 56 171 20 253 220 212 158 90 149 78 165 0 160 54 145 180 88 197 218 54 91 69 179 169 222 34 180 88 48 193 162 139 90 233 231 234 20 101 112 27 17 232 238 174 81 253 74 239 96 52 26 227 227 227 11 10 10 0 64 163 209 156 56 113 98 232 208 161 242 104 109 109 109 81 81 81 92 92 156 94 175 255 181 209 84 56 80 115 76 36 66 40 7 0 64 41 16 10 132 82 12 20 83 4 20 176 68 120 142 139 11 208 15 51 184 186 168 29 76 55 139 248 138 209 114 177 193 84 84 219 116 233 39 83 97 141 241 199 26 227 55 23 106 199 245 247 186 59 212 199 77 163 236 245 119 80 171 213 222 222 222 172 237 229 229 229 225 225 113 93 50 138 139 167 77 155 118 233 210 165 152 152 152 3 7 14 200 108 183 66 53 45 150 102 139 56 196 215 181 227 16 255 210 75 47 181 235 106 48 9 245 109 2 80 0 66 9 5 2 148 16 74 8 1 66 36 10 10 158 191 39 204 43 170 159 94 221 137 193 81 242 156 151 86 117 135 151 115 140 191 251 64 47 103 30 113 77 86 169 166 197 86 88 109 60 85 221 236 164 84 4 187 105 81 175 158 169 10 133 98 216 176 97 165 165 165 30 30 30 175 190 250 106 92 92 156 60 180 110 221 186 111 190 249 134 73 168 191 191 255 168 81 163 110 113 175 83 149 77 211 223 251 254 173 195 103 21 74 197 232 254 94 221 203 166 2 33 66 1 81 10 0 132 82 76 41 161 132 16 66 41 37 148 27 19 234 25 230 169 235 201 198 90 37 31 225 167 143 240 211 95 106 52 101 149 54 252 167 236 167 146 250 182 141 217 23 138 174 182 44 24 17 236 164 224 111 226 101 246 238 221 155 145 145 49 96 192 128 39 159 124 210 213 245 186 116 68 71 71 31 58 116 168 35 255 160 65 131 228 246 144 33 67 110 253 203 189 123 172 228 92 89 61 240 220 230 220 243 11 70 133 185 254 82 59 29 200 102 163 69 42 55 90 232 53 28 41 161 0 128 8 161 130 68 6 120 234 198 6 187 223 232 19 184 107 84 49 254 110 209 254 122 10 244 74 179 249 72 69 83 173 201 54 42 208 93 113 131 182 162 168 168 104 234 212 169 199 143 31 207 202 202 210 106 181 19 38 76 232 118 74 68 68 4 198 88 20 197 231 158 123 110 206 156 57 232 214 148 194 44 226 245 135 206 212 180 217 64 193 217 8 189 111 176 33 200 77 215 141 108 170 120 68 9 96 66 41 101 234 14 148 18 140 41 7 40 210 207 229 166 31 37 216 77 187 244 174 176 241 33 222 219 10 42 246 20 215 184 169 249 21 99 6 222 40 154 70 163 145 181 243 243 243 123 50 69 163 209 188 242 202 43 189 117 164 152 5 177 180 161 13 56 4 0 22 65 106 182 8 237 24 28 72 135 74 193 81 10 152 65 73 128 18 74 8 136 152 120 104 85 126 46 234 91 124 160 97 6 253 43 247 69 172 142 191 35 183 188 33 191 178 233 134 230 78 153 50 229 174 187 238 2 0 189 94 191 96 193 130 94 60 121 77 38 83 79 216 68 66 141 86 1 16 2 14 129 77 108 48 217 186 63 55 213 28 34 148 96 66 56 74 8 112 132 2 37 4 19 234 227 162 86 245 134 231 168 224 80 98 132 127 184 143 139 85 36 55 52 209 197 197 101 215 174 93 185 185 185 3 6 12 136 137 137 233 45 40 83 82 82 238 186 235 174 123 239 189 183 123 119 18 128 67 8 51 159 18 99 171 136 123 128 166 130 87 32 176 97 66 17 37 4 99 74 1 40 165 196 91 215 155 14 99 184 143 235 77 204 242 245 245 77 72 72 232 197 199 88 181 106 85 106 106 170 67 11 230 144 174 123 231 10 94 171 84 116 143 166 70 201 41 57 132 9 69 8 97 74 49 165 148 130 68 169 139 147 226 87 114 122 153 161 0 0 132 144 90 173 134 223 138 86 175 94 157 154 154 170 82 169 220 221 123 100 90 121 132 116 106 69 171 85 0 2 72 173 244 114 86 247 192 67 226 144 74 193 91 9 40 16 197 64 49 32 66 8 199 113 58 149 99 52 247 236 217 83 92 92 204 218 19 38 76 96 71 91 71 218 186 117 107 99 99 35 107 47 88 176 192 199 199 7 0 126 248 225 135 157 59 119 158 57 115 166 185 185 25 33 196 243 124 116 116 244 148 41 83 238 185 231 158 142 43 180 182 182 126 248 225 135 236 140 139 143 143 31 61 122 180 60 180 115 231 206 11 23 46 0 128 135 135 71 82 82 146 179 179 51 235 175 168 168 248 215 191 254 5 0 28 199 37 36 36 132 133 133 177 126 65 16 150 47 95 254 206 59 239 0 0 33 100 203 150 45 3 6 12 176 247 4 166 77 155 230 80 107 13 174 154 86 179 13 16 162 0 212 145 232 58 160 253 231 235 254 47 179 228 213 172 146 13 89 37 235 50 75 94 62 116 110 67 230 185 38 139 224 144 249 189 247 222 147 87 139 141 141 149 36 169 35 79 118 118 246 117 227 30 28 220 214 214 38 8 194 154 53 107 220 220 220 28 198 54 79 61 245 148 40 138 237 22 41 47 47 151 37 247 197 23 95 180 31 146 79 61 95 95 223 202 202 74 185 255 224 193 131 242 178 187 118 237 98 157 86 171 117 238 220 185 93 200 224 140 25 51 28 190 105 171 85 140 76 221 7 43 182 193 138 109 218 231 62 253 207 229 186 118 12 142 173 138 94 205 19 44 17 138 137 132 9 150 8 145 16 33 138 78 156 181 199 30 123 76 142 49 126 252 241 199 3 7 14 116 228 121 235 173 183 228 246 218 181 107 117 58 93 122 122 250 250 245 235 155 155 155 59 50 219 108 182 180 180 180 141 27 55 182 247 63 56 78 70 95 163 209 216 15 201 194 168 215 235 237 83 30 74 229 181 64 150 231 121 214 110 106 106 154 53 107 214 246 237 219 187 210 104 158 239 44 30 9 114 215 2 161 0 160 83 242 250 14 81 178 162 19 52 21 0 32 225 107 145 37 198 20 35 210 197 222 79 63 253 244 131 15 62 200 142 191 77 155 54 221 127 255 253 246 175 84 84 84 180 103 207 30 214 142 138 138 154 57 115 38 0 156 57 115 134 245 120 120 120 140 25 51 38 52 52 84 20 197 211 167 79 231 230 230 178 254 215 95 127 253 193 7 31 12 14 14 238 221 179 242 244 233 211 173 173 173 147 39 79 174 171 171 99 137 18 142 227 98 99 99 237 163 251 145 35 71 58 156 203 113 200 224 234 4 132 0 112 174 26 149 159 139 182 71 104 186 58 41 121 14 9 18 166 20 8 0 38 84 100 222 124 39 52 117 234 212 17 35 70 28 61 122 20 0 190 253 246 219 3 7 14 76 153 50 69 30 125 243 205 55 45 22 11 107 207 157 59 151 229 114 216 247 79 74 74 122 246 217 103 163 162 162 174 57 116 162 56 127 254 252 29 59 118 0 64 117 117 245 209 163 71 123 29 205 248 248 248 248 248 120 0 248 238 187 239 216 249 160 86 171 223 122 235 173 17 35 70 244 100 122 160 155 14 8 5 68 221 52 42 87 39 101 247 222 59 0 56 171 21 106 158 19 37 42 17 42 96 98 195 216 42 17 145 118 42 158 78 78 78 79 63 253 180 252 231 63 254 241 15 185 93 86 86 150 145 145 113 237 81 2 3 31 123 236 49 89 109 95 126 249 229 29 59 118 200 80 50 197 92 188 120 177 86 171 189 33 167 250 22 61 118 74 105 91 91 91 15 103 245 115 213 2 2 160 16 236 166 83 114 168 71 104 186 168 21 78 42 222 74 136 72 136 136 169 68 168 32 98 161 75 103 251 129 7 30 144 173 121 78 78 78 94 94 30 107 111 223 190 189 190 190 158 181 23 46 92 232 233 233 201 218 201 201 201 47 190 248 98 199 117 60 61 61 229 195 238 191 149 244 237 130 2 220 180 188 86 13 0 81 253 28 216 79 174 179 112 197 87 171 66 4 35 130 41 145 40 150 68 73 180 137 82 23 219 168 84 170 149 43 87 50 253 181 88 44 233 233 233 0 208 220 220 44 91 124 111 111 239 71 30 121 68 230 151 157 21 123 202 206 206 222 188 121 243 175 42 146 183 72 33 30 58 79 157 26 40 29 234 231 214 163 108 49 35 31 103 53 38 132 32 68 8 37 132 98 73 106 179 137 93 239 52 99 198 140 161 67 135 22 22 22 50 119 228 236 217 179 185 185 185 21 21 21 108 116 222 188 121 65 65 65 14 93 180 179 103 207 126 253 245 215 59 119 238 60 117 234 148 36 73 240 59 166 1 158 206 62 58 167 186 250 214 161 126 250 27 64 211 207 213 9 83 160 148 18 74 36 74 4 137 180 88 187 65 83 169 84 46 91 182 108 225 194 133 204 158 188 240 194 11 213 213 213 215 124 11 173 118 209 162 69 29 167 228 228 228 108 220 184 113 239 222 189 132 16 0 232 223 191 255 157 119 222 185 111 223 190 158 31 100 191 49 41 56 46 170 159 91 109 171 197 199 89 125 3 104 26 92 157 116 106 101 147 89 0 74 37 76 109 152 52 89 108 221 110 54 107 214 172 215 95 127 157 121 63 187 118 237 146 251 31 122 232 161 144 144 144 118 204 219 183 111 95 186 116 105 107 107 43 115 74 150 44 89 50 117 234 212 154 154 26 123 151 251 119 72 143 196 13 8 116 215 185 107 213 61 61 55 1 192 89 165 244 115 113 178 72 88 32 20 83 42 98 82 223 102 233 118 39 55 55 183 101 203 150 117 76 50 46 94 188 184 93 166 54 51 51 115 225 194 133 12 202 228 228 228 220 220 220 71 31 125 212 211 211 211 102 179 49 57 253 45 233 134 178 200 147 7 25 94 249 83 180 195 9 157 162 201 33 8 212 107 176 68 40 161 20 3 193 80 219 108 238 201 102 115 231 206 29 60 120 176 125 207 164 73 147 98 99 99 219 177 165 166 166 90 173 86 22 218 111 216 176 65 182 227 191 125 237 9 33 164 183 190 31 215 181 253 82 114 28 38 148 18 74 129 54 180 90 173 98 247 38 66 171 213 62 255 252 243 246 145 82 114 114 114 71 95 175 188 188 92 14 219 237 61 33 39 39 167 235 199 144 66 241 235 129 40 127 54 65 16 78 156 56 97 63 212 210 210 82 81 81 209 217 119 109 117 148 39 238 30 205 64 55 173 179 147 194 44 17 27 0 1 84 103 178 53 152 172 61 121 80 150 31 186 166 23 147 39 119 188 41 228 56 78 70 48 59 59 251 242 229 203 242 80 86 86 22 147 89 0 168 171 171 3 128 130 130 2 150 175 235 93 10 12 12 148 131 253 180 180 180 143 62 250 168 184 184 56 55 55 55 61 61 125 194 132 9 127 255 251 223 29 170 255 182 252 203 35 54 238 15 91 247 229 234 127 23 88 37 210 83 43 4 0 94 58 181 167 206 169 220 104 86 114 8 35 174 222 44 213 182 88 2 220 156 187 85 156 13 27 54 200 231 209 194 133 11 59 58 225 26 141 38 38 38 166 168 168 136 5 75 211 166 77 155 61 123 182 82 169 204 206 206 102 23 182 140 210 211 211 79 158 60 121 228 200 145 243 231 207 247 58 154 195 134 13 27 55 110 28 179 120 117 117 117 243 230 205 243 246 246 54 153 76 102 179 25 0 198 140 25 211 113 202 217 186 150 5 159 230 73 130 4 28 122 101 111 65 184 175 254 225 216 144 158 202 38 66 48 208 219 89 34 32 82 74 16 152 68 233 98 67 247 142 203 247 223 127 47 7 66 227 199 143 159 62 125 186 67 182 228 228 100 95 95 95 57 45 178 118 237 218 213 171 87 127 243 205 55 225 225 225 178 200 84 84 84 124 252 241 199 102 179 153 169 60 165 84 118 236 5 225 23 55 92 178 56 155 205 102 123 13 197 24 203 141 118 158 172 82 169 124 237 181 215 252 253 253 229 158 250 250 122 6 165 253 68 123 106 178 8 146 32 129 130 7 158 3 74 74 234 91 111 64 54 1 32 194 224 42 21 176 219 116 66 120 84 88 219 252 96 119 89 244 55 222 120 67 126 238 69 139 22 117 102 46 35 34 34 246 237 219 183 102 205 154 211 167 79 219 108 54 141 70 19 24 24 248 240 195 15 39 37 37 189 253 246 219 91 183 110 109 105 105 209 235 245 195 135 15 95 181 106 21 147 110 181 90 61 97 194 4 150 114 110 231 111 69 69 69 177 235 204 160 160 32 251 236 189 187 187 59 75 42 115 28 215 177 208 35 42 42 42 51 51 51 37 37 37 47 47 207 104 52 82 74 21 10 133 193 96 152 56 113 226 242 229 203 29 60 179 175 126 176 191 251 185 178 122 0 112 114 213 76 31 26 208 94 254 186 182 161 70 171 184 108 111 65 101 147 137 3 34 74 196 83 171 252 96 246 104 119 109 167 23 68 199 142 29 27 51 102 12 251 176 49 49 49 217 217 217 114 230 177 51 186 112 225 130 217 108 118 113 113 9 13 13 149 59 43 42 42 234 235 235 253 252 252 236 101 231 215 163 171 87 175 214 214 214 178 248 120 208 160 65 93 228 7 254 83 86 159 158 89 236 164 228 103 15 15 153 26 238 127 99 104 82 74 55 100 157 223 83 92 165 225 57 137 96 66 232 107 127 138 26 55 192 167 51 254 71 31 125 116 219 182 109 242 213 5 139 139 110 31 226 186 117 107 227 252 221 121 196 9 152 82 130 218 108 56 167 236 167 206 152 11 11 11 229 248 39 36 36 36 49 49 17 110 51 234 62 229 21 101 208 187 107 148 86 76 69 0 202 115 71 43 27 27 205 130 67 206 15 62 248 64 142 175 87 172 88 97 159 205 238 67 243 231 244 135 139 83 136 183 174 149 136 102 192 18 135 10 27 90 143 87 53 118 100 187 124 249 178 172 227 253 250 245 235 221 91 239 63 14 154 0 48 190 191 143 72 64 34 64 40 21 8 205 40 174 238 200 243 230 155 111 54 53 53 201 193 101 64 64 128 72 168 72 110 175 159 40 244 40 116 27 21 224 238 161 81 93 53 219 148 8 1 208 236 138 134 194 171 198 72 223 235 9 190 234 234 234 47 190 248 130 181 61 61 61 89 226 227 76 173 81 167 82 220 225 117 221 166 167 164 164 124 254 249 231 127 24 236 148 74 101 102 102 166 125 101 67 143 208 12 208 107 162 251 185 125 94 84 165 83 114 20 104 101 139 229 253 130 242 244 251 34 101 134 45 91 182 200 89 225 164 164 164 128 128 0 0 56 95 103 28 242 203 242 152 242 242 114 22 255 252 97 168 93 68 208 35 52 121 132 38 246 247 222 89 92 99 193 64 16 80 14 125 82 84 181 32 166 127 164 143 43 11 37 93 93 93 151 44 89 130 16 82 169 84 44 43 76 40 61 219 98 11 239 247 139 245 85 170 107 142 234 192 129 3 153 251 205 113 28 165 180 178 178 210 30 101 131 193 48 126 252 120 127 127 255 230 230 230 147 39 79 158 62 125 154 37 83 198 141 27 71 41 205 207 207 111 108 108 84 169 84 241 241 241 132 144 19 39 78 24 141 70 119 119 247 184 184 56 132 208 225 195 135 189 189 189 217 47 51 216 226 0 80 90 90 122 241 226 197 65 131 6 177 27 80 214 95 87 87 247 195 15 63 200 30 200 192 129 3 237 83 115 103 207 158 173 168 168 24 54 108 152 193 96 96 83 4 65 40 46 46 102 110 169 252 58 237 99 19 218 51 170 110 181 6 164 31 128 148 61 176 126 47 172 223 11 107 119 255 57 35 191 11 126 147 40 45 217 127 186 220 104 182 239 124 226 137 39 228 0 220 190 223 104 52 202 161 253 164 73 147 74 74 74 228 33 171 213 250 183 191 253 13 0 252 252 252 154 155 155 41 165 243 230 205 3 128 251 239 191 159 49 36 37 37 1 192 95 254 242 23 198 236 228 228 180 98 197 138 118 15 147 154 154 202 20 200 190 211 98 177 188 251 238 187 44 102 77 73 73 105 55 229 169 167 158 2 128 47 191 252 210 190 179 180 180 116 245 234 213 50 130 42 149 170 190 190 190 251 90 15 7 169 120 103 245 228 16 111 176 137 128 49 96 2 8 246 156 169 252 180 168 170 51 254 42 163 181 222 100 243 211 57 46 209 98 215 235 71 142 28 153 61 123 118 106 106 170 139 139 203 243 207 63 63 124 248 112 79 79 207 29 59 118 132 133 133 189 255 254 251 113 113 113 139 22 45 194 24 167 164 164 60 244 208 67 181 181 181 95 125 245 21 203 86 0 0 187 19 7 128 232 232 104 22 119 1 64 70 70 134 213 106 101 9 39 150 76 153 49 99 70 66 66 194 71 31 125 36 111 122 248 240 225 153 51 103 174 89 179 70 165 82 61 254 248 227 119 223 125 183 28 230 159 60 121 50 33 33 97 250 244 233 137 137 137 172 156 130 133 237 135 14 29 74 72 72 216 176 97 131 191 191 255 250 245 235 87 175 94 125 75 54 157 81 226 16 3 226 17 16 2 4 3 0 193 120 213 129 31 47 53 57 78 33 103 94 174 7 10 93 215 123 94 184 112 225 139 47 190 88 181 106 21 187 34 14 12 12 188 239 190 251 124 124 124 74 74 74 22 45 90 116 242 228 201 119 222 121 103 235 214 173 0 48 103 206 28 0 96 201 148 216 216 88 149 74 21 23 23 199 226 215 240 240 112 119 119 119 86 223 158 147 147 3 63 223 27 99 140 175 94 189 90 87 87 87 93 93 93 90 90 42 111 122 238 220 185 221 187 119 167 165 165 177 180 191 125 10 85 16 132 186 186 186 250 250 250 43 87 174 84 85 85 201 138 127 238 220 185 93 187 118 173 89 179 230 175 127 253 43 0 44 94 188 184 179 154 186 27 64 115 108 160 215 48 95 87 176 137 215 0 69 244 74 157 113 193 151 249 141 29 10 150 9 133 140 226 202 254 110 154 174 23 12 11 11 155 61 123 246 166 77 155 124 124 124 76 38 211 177 99 199 238 188 243 78 22 83 201 41 156 195 135 15 3 64 104 104 168 78 167 203 204 204 52 155 205 145 145 145 145 145 145 49 49 49 135 14 29 202 205 205 29 61 122 116 84 84 212 240 225 195 45 22 139 125 210 55 52 52 244 248 241 227 71 142 28 201 203 203 99 242 203 104 226 196 137 111 188 241 70 86 86 150 94 175 207 202 202 146 11 117 0 96 244 232 209 57 57 57 71 142 28 57 118 236 152 253 79 14 100 196 119 239 222 13 0 94 94 94 17 17 17 55 239 33 49 114 81 43 102 12 50 20 150 53 128 92 227 160 224 178 206 213 252 239 167 199 222 157 57 60 72 127 189 40 231 139 51 149 223 95 168 125 44 170 155 162 151 177 99 199 142 29 59 150 57 88 107 215 174 173 169 169 97 55 10 246 69 85 44 59 39 138 34 207 243 37 37 37 37 37 37 17 17 17 243 231 207 247 240 240 248 238 187 239 120 158 31 59 118 236 204 153 51 245 122 125 126 126 190 189 41 187 114 229 202 51 207 60 163 80 40 40 165 172 24 145 81 80 80 208 210 165 75 1 224 252 249 243 115 230 204 97 18 202 168 160 160 32 45 45 77 146 36 140 177 125 2 91 38 157 78 199 68 216 126 214 77 162 9 0 73 81 193 105 57 23 218 172 18 112 0 148 178 10 240 131 63 86 78 254 169 237 217 9 131 167 12 242 83 112 220 215 231 170 147 255 93 8 4 13 241 238 38 123 148 151 151 183 121 243 102 81 20 143 29 59 118 229 202 21 38 137 43 87 174 28 53 106 148 193 96 168 169 169 1 0 22 83 21 22 22 182 180 180 176 68 125 84 84 212 252 249 243 41 165 121 121 121 76 169 89 53 206 169 83 167 108 182 235 23 12 13 13 13 14 125 219 143 63 254 56 43 43 235 159 255 252 103 88 88 88 98 98 162 125 241 94 121 121 185 195 218 57 89 81 152 166 95 186 116 73 174 87 189 37 52 239 240 114 254 159 59 124 63 59 126 9 88 101 44 179 110 10 116 190 166 233 241 207 142 250 186 106 17 7 181 45 86 144 72 136 159 62 204 211 185 139 100 10 123 172 79 62 249 228 23 167 109 102 230 241 227 199 71 142 28 249 213 87 95 125 248 225 135 35 71 142 76 76 76 52 153 76 239 191 255 62 99 56 120 240 224 242 229 203 213 106 117 93 93 221 169 83 167 92 93 93 155 155 155 89 25 162 156 180 103 139 7 7 7 111 218 180 137 231 121 158 231 247 237 219 183 127 255 126 214 79 8 217 182 109 91 116 116 244 147 79 62 153 150 150 118 241 226 197 131 7 15 178 161 152 152 152 205 155 55 19 66 148 74 229 39 159 124 146 147 147 195 250 227 227 227 83 83 83 71 142 28 57 126 252 120 179 217 188 106 213 170 118 185 234 155 68 19 0 146 162 131 50 242 47 99 17 3 143 126 174 2 71 0 20 48 185 218 216 10 28 2 14 1 33 195 124 92 92 212 157 254 8 80 16 4 139 197 210 49 191 221 210 210 146 152 152 184 113 227 198 89 179 102 177 159 164 21 22 22 38 39 39 203 197 180 103 206 156 169 170 170 242 240 240 248 246 219 111 45 22 139 40 138 185 185 185 247 220 115 143 201 100 98 21 122 236 88 176 88 44 58 157 110 229 202 149 172 167 177 177 113 255 254 253 108 83 118 152 188 240 194 11 209 209 209 99 198 140 97 183 64 109 109 109 22 139 197 96 48 200 215 215 197 197 197 57 57 57 86 171 213 98 177 12 30 60 56 34 34 162 165 165 37 35 35 99 243 230 205 246 149 189 55 150 223 236 72 109 54 41 254 141 3 39 75 235 64 197 255 12 37 0 80 0 4 8 128 227 0 33 176 138 107 31 136 121 105 114 100 187 185 139 23 47 222 178 101 11 59 13 213 106 181 32 8 242 205 65 59 234 223 191 191 193 96 48 26 141 165 165 165 246 250 11 0 206 206 206 10 133 194 98 177 176 126 182 20 33 132 29 5 44 69 223 174 86 214 106 181 90 173 214 118 155 42 149 74 157 78 167 84 42 89 145 184 92 152 39 251 112 54 155 77 171 213 202 17 7 198 184 221 113 169 82 169 170 170 170 188 188 188 110 94 54 157 213 138 121 35 195 78 94 168 5 204 1 32 64 20 16 2 64 64 41 32 0 32 64 17 167 226 71 4 120 118 177 136 197 98 145 43 58 29 82 89 89 89 89 89 153 227 207 249 203 162 154 142 75 217 108 182 118 31 192 33 167 40 138 246 149 205 14 149 215 108 54 119 246 189 111 213 67 146 41 33 58 56 192 75 15 34 6 66 228 95 187 1 1 192 24 48 6 65 236 231 172 113 88 144 39 223 133 253 49 72 16 132 118 154 125 147 255 215 227 185 221 249 175 238 61 5 106 5 32 244 179 113 231 0 81 32 0 22 225 129 225 33 123 150 78 114 40 113 87 175 94 253 195 160 137 16 138 141 141 181 247 255 111 18 205 179 53 205 163 95 249 202 104 182 129 146 7 66 1 1 240 60 251 143 10 96 178 189 253 248 196 39 238 14 135 219 143 110 178 120 119 136 193 237 161 17 161 96 19 175 169 185 132 65 176 129 96 3 139 205 199 77 247 167 200 32 184 45 233 230 75 161 159 156 28 225 229 166 1 139 13 36 12 146 4 86 1 172 34 152 108 127 142 11 9 242 116 238 67 243 198 104 160 175 254 153 41 209 32 136 32 138 32 97 144 8 8 216 213 77 187 104 226 80 184 93 233 150 202 244 87 76 138 252 243 168 129 96 52 131 77 4 9 131 85 88 126 95 84 76 176 247 109 139 230 173 254 175 174 198 54 235 146 119 191 251 44 167 88 193 243 243 238 141 76 159 23 175 81 41 250 208 188 133 187 17 76 10 202 235 21 28 138 10 246 70 8 193 109 76 125 255 71 174 15 205 62 52 251 208 236 163 62 52 255 123 244 255 3 0 9 33 97 176 122 213 141 185 0 0 0 0 73 69 78 68 174 66 96 130) asByteArray ].
	^ cache at: 1! !

!ITFileLibrary methodsFor: 'uploaded' stamp: ''!
mainCss
	^ '/* general settings for all components */
body {
	margin: 0px;
	font: 11px Lucida Grande, Verdana, Arial, Helvetica, sans serif;
}

:link, :visited {
	color: #c00;
	text-decoration: none;
	border-bottom: 1px dotted #bbb;
}

:link:hover, :visited:hover {
	background-color: #eee;
	color: #555;
}

.content {
	width: 700px;
	max-width: 100%;
	margin: 5px;
}

fieldset {
	margin: 5px;
	color: #bbb;
	border: 1px solid #bbb;
	font-weight: bold;
}

fieldset ul li {
	color: #000;
	font-weight: normal;
}

/* links/anchors with images */
:link img, :visited img{
	color: #fff;
	background-color: #fff
	
	text-decoration: none;
	border: none;
	
	display: block;
}

:link:hover img, :visited:hover img{
	color: #09c;
	background-color: #09c
}

/* a div button */
.button {
	margin: 10px auto;
	background-color: #eee;
	border: 1px dotted #bbb;
	width: 100px;
	padding: 5px;
	text-align: center;
}

/* login menu */
#login_menu {
	text-align: right;
	color: #bbb;
	margin: 5px;
	padding: 5px;
}

#login_menu .text {
	color: #000;
}

/* main menu */
#main_menu {
	margin: .66em 5px;
	padding: .2em 0;
	border: 1px solid #000;
	background-color: #f7f7f7;
}

#main_menu ul {
	margin: 0;
	font-size: 10px;
	text-align: right;
	
	list-style: none;
}

#main_menu li {
	border-right: none;
	padding: .25em 0;
	
	display: inline;
	white-space: nowrap;
}

#main_menu :link, #main_menu :visited {
	color: #000;
	background-color: #fff;
	border-left: 1px solid #555;
	border-bottom: none;
	border-right: 1px solid #fff;
	padding: .2em 20px;
}

#main_menu :link:hover, #main_menu :visited:hover {
	background-color: #ccc;
	border-right: 1px solid #ddd;
}

#main_menu .active :link, #main_menu .active :visited {
	font-weight: bold;
	color: #eee;
	border-right: 1px solid #000;
	border-top: none;
	background-color: #333;
}

#main_menu .active :link:hover, #main_menu .active :visited:hover {
	border-right: 1px solid #000;
}

/* side menu (left) */
#tab_menu {
	border: 1px solid black;
	border-color: gray black black gray;
	margin: 2em 5px;
	padding: .5em 0 0;
	float: left;
	width: 12em;
}

#tab_menu ul { 
	list-style: none;
	margin: 0 0 .5em;
	padding: 0;
}

#tab_menu li {
	margin: 0;
	padding: 0.1em .5em;
	font-weight: bold;
}
#tab_menu li li { margin: 0 -0.5em; padding-left: 1.5em; font-weight: normal }
#tab_menu li li.active { background: #ccb; }
#tab_menu :link, .tabs :visited { border: none; display: block }
#tab_menu :link:hover, .tabs :visited:hover { background: transparent }

.content_with_menu {
	padding: 0.4em 2em;
	margin-left: 12em;
	min-height: 300px;
	width: 400px;
	max-width: 100%
}

/* detailed view */
#issue {
 background: #ffd;
 border: 1px outset #996;
 margin-top: 1em;
 padding: .5em 1em;
 position: relative;
}

#issue h2 {
	margin: 0 0 .8em 0;
}

#issue table.details {
 border-top: 1px solid #dd9;
 border-collapse: collapse;
 table-layout: fixed;
 width: 100%;
}

#issue table.details tr { 
	border-bottom: 1px dotted #eed;
}

#issue table.details td, #issue table.details th {
 font-size: 80%;
 padding: .5em 1em;
 vertical-align: top;
}

#issue table.details th {
 color: #663;
 font-weight: normal;
 text-align: left;
 width: 20%;
}

#issue table.details td {
	width: 30%;
}

#issue table.details .description {
	border-top: 1px solid #dd9;
}

#issue .description h3 {
 border-bottom: 1px solid #dd9;
 color: #663;
 font-size: 100%;
 font-weight: normal;
}

.comment {
	background: #ffc;
	padding: 1em;
	border: 1px outset #996;
	margin-bottom: 1em;
}

.comment h3 {
 border-bottom: 1px solid #d7d7d7;
 color: #999;
 font-size: 100%;
 font-weight: normal;
}

/* drag & droppable lists */

fieldset.drag_list {
	float: left;
	width: 170px;
}

fieldset.drag_list ul {
	min-height: 50px;
	color: #000;
	
	list-style: none;
	padding: 0;
	margin: 0;
}

fieldset.drag_list ul li {
	margin: 4px;
	padding: 2px;
	cursor: move;
	font-weight: normal;
	text-align: center;
}

fieldset.drag_list ul li.attached {
	background-color: #6c6;
  	border: 1px solid #060;
}
fieldset.drag_list ul li.available {
	background-color: #ddd;
  	border: 1px dashed #000;
}

/* list issues table */

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block
}

.list_issues {
	border-spacing: 0px;
}

.list_issues thead th {
	background-color: #eee;
	text-transform: capitalize;
	vertical-align: middle;
	
	border-top: 1px solid #ddd;
	border-left: 1px solid #ddd;
	border-bottom: 1px solid #5F9EA0;
	
	padding: .3em 1em;
}

.list_issues tbody tr {
	text-align: center;
}

.list_issues tbody td {
	padding: .1em .5em;
	border-bottom: 1px solid #5F9EA0;
	border-left: 1px solid #5F9EA0;
}

.list_issues tbody td :link, .list_issues tbody td :visited {
	display: block;
	font-weight: bold;
	border: none;
	background: none;
}

.list_issues tbody tr:hover {
	background: #ff0;
}

.list_issues tr.odd {
	background-color: #B0E0E6;
}

.list_issues tr.even {
	background-color: #ADD8E6;
}


/*
#main_menu a.active {
	color: #fff;
	background-color: #000;
}

#main_menu a:hover {
	color: #fff;
	background-color: #000;
}
*/
/* a simple button */
/* a.button, a.button:hover {
	text-decoration: none;		
	color: #000
}
	
a.button:hover {
	color: #fff;
}
	
span.button, span.button:hover {
		margin: 5px;
		padding: 5px;
		font-size: 10px;
		font-family: Verdana,Arial;
		border-width: 1px;
		border-style: none, none, solid, none;
		border-color: #000;
		text-align: center;
		background-color: #fff;		
	}
	
div span.button:hover {
	background-color: #000;
}*/

/* older stuff */

	
	div.security_container {
		padding: 10px;
		border-width: 1px;
		border-style: solid;
		width: 500px;
	}
	
	div.security_menu {
		margin: 0px;
	}
	'! !

!ITFileLibrary methodsFor: 'accessing' stamp: 'afk 1/8/2008 22:20'!
selectorsToInclude
	"Appends a unique style to all used components."
	
	"^#(#mainCss)."
	^ #(#hpidesignCss).! !

DescriptorSystem subclass: #ITGlorpDescriptorSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDescriptorSystem commentStamp: 'mt 12/30/2007 14:38' prior: 0!
I'm the descriptor system that should be used in every new ITGlorpSession. I'm responsible for all mappings between the IT-Classes and the relevant tables on the SQL server.

You should modify #allTableNames and #constructAllClasses whenever you want to add new objects of the domain space for usage here.

#tableNamed<TABLE_NAME>    ... descriptions about the table structure on the server
#descriptorFor<CLASS_NAME> ... defines mappings between classes and tables
#classModelFor<CLASS_NAME> ... defines types (classes) for symbols (attributes) used by mappings

The relational schemes can be found in the comments for any #tableNamed<TABLE_NAME>. Primary keys are underlined, e.g. 

SCHEME(ATTRIBUTE_A, ATTRIBUTE_B)
                    -----------!

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'afk 1/6/2008 13:22'!
allTableNames
	"Here are all tables listed which hold all persistent data."
	
	^ #('ISSUE' 'COMMENT' 'DETAIL' 'USERS' 'ROLE' 'USERROLE' 'ROLEPRIVILEGE' 'TAGS')! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:54'!
classModelForITIssue: aClassModel
	"The unique ID can be up to 1.073.741.823 - should be enough.
	 -> SmallInteger maxVal
	 4-Byte-Integers (signed) are used in the database"
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #comments collection: OrderedCollection  of: ITIssueComment.
	aClassModel newAttributeNamed: #details collection: OrderedCollection  of: ITIssueDetail.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 1/5/2008 11:16'!
classModelForITIssueComment: aClassModel
	"See #classModelForITIssue for explanation of ID."
	
	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #content type: String.
	aClassModel newAttributeNamed: #timestamp type: TimeStamp.
	aClassModel newAttributeNamed: #user type: ITUser.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 12/14/2007 09:55'!
classModelForITIssueDetail: aClassModel
	"See #classModelForITIssue for explanation of ID."

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #tag type: String.
	aClassModel newAttributeNamed: #content type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mt 1/5/2008 23:28'!
classModelForITRole: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #rolename type: String.
	aClassModel newAttributeNamed: #privileges collection: Set of: String.
	aClassModel newAttributeNamed: #users collection: Set of: ITUser.
	! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'afk 1/9/2008 19:02'!
classModelForITTag: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #name type: String.
	aClassModel newAttributeNamed: #description type: String.! !

!ITGlorpDescriptorSystem methodsFor: 'class models' stamp: 'mig 12/17/2007 14:57'!
classModelForITUser: aClassModel

	aClassModel newAttributeNamed: #id type: SmallInteger.
	aClassModel newAttributeNamed: #username type: String.
	aClassModel newAttributeNamed: #passwordHash type: String.
	aClassModel newAttributeNamed: #roles collection: Set of: ITRole.
! !

!ITGlorpDescriptorSystem methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:23'!
constructAllClasses
	"Here are all classes listed which are necessary for the mappings."
	
	^ (super constructAllClasses)
		add: ITIssue;
		add: ITIssueComment;
		add: ITIssueDetail;
		add: ITUser;
		add: ITRole;
		add: ITTag;
		yourself! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/7/2008 21:25'!
descriptorForITIssue: aDescriptor
	"Every issue contains a unique ID, a collection of comments and a collection of details."
	
	| issueTable commentTable detailTable |
	issueTable   := self tableNamed: 'ISSUE'.
	commentTable := self tableNamed: 'COMMENT'.
	detailTable  := self tableNamed: 'DETAIL'.

	aDescriptor table: issueTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (issueTable fieldNamed: 'ID').

	(aDescriptor newMapping: ToManyMapping)
		attributeName: #comments;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to: (commentTable	fieldNamed: 'ISSUE_ID')
		); beExclusive.
	
	(aDescriptor newMapping: ToManyMapping)
		attributeName: #details;
		join: (Join
			from: (issueTable fieldNamed: 'ID')
			to:  (detailTable fieldNamed: 'ISSUE_ID')
		); beExclusive.! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/5/2008 11:04'!
descriptorForITIssueComment: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'COMMENT'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').
	(aDescriptor newMapping: DirectMapping) from: #timestamp to: (table fieldNamed: 'TIMESTAMP').

	(aDescriptor newMapping: OneToOneMapping)
		attributeName: #user.! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 12/14/2007 09:47'!
descriptorForITIssueDetail: aDescriptor
	"Every detail contains a unique ID, a tag and its content."

	| table |
	table := self tableNamed: 'DETAIL'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #tag to: (table fieldNamed: 'TAG').
	(aDescriptor newMapping: DirectMapping) from: #content to: (table fieldNamed: 'CONTENT').! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/7/2008 21:44'!
descriptorForITRole: aDescriptor
	
	| roleTable userRoleTable rolePrivilegeTable |
	roleTable := self tableNamed: 'ROLE'.
	userRoleTable := self tableNamed: 'USERROLE'.
	rolePrivilegeTable := self tableNamed: 'ROLEPRIVILEGE'.

	aDescriptor table: roleTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (roleTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #rolename to: (roleTable fieldNamed: 'ROLENAME').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #users;
		referenceClass: ITUser;
		useLinkTable;
		mappingCriteria: (Join
			from: (roleTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'ROLE_ID')
		).
	
	(aDescriptor newMapping: DirectToManyMapping)
		attributeName: #privileges;
		resultField: (rolePrivilegeTable fieldNamed: 'PRIVILEGE');
		writeTheOrderField;
		orderBy: [:each | (each getTable: rolePrivilegeTable) getField: 'PRIVILEGE_ORDER'];
		join: (Join from: (roleTable fieldNamed: 'ID') to: (rolePrivilegeTable fieldNamed: 'ROLE_ID')).! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'afk 1/9/2008 19:02'!
descriptorForITTag: aDescriptor
	"Every comment contains a unique ID and its content - should be text."
	
	| table |
	table := self tableNamed: 'TAGS'.
	aDescriptor table: table.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (table fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #name to: (table fieldNamed: 'TAGNAME').
	(aDescriptor newMapping: DirectMapping) from: #description to: (table fieldNamed: 'TAGDESCR').
! !

!ITGlorpDescriptorSystem methodsFor: 'descriptors' stamp: 'mt 1/7/2008 21:43'!
descriptorForITUser: aDescriptor
	
	| userTable userRoleTable |
	userTable   := self tableNamed: 'USERS'.
	userRoleTable := self tableNamed: 'USERROLE'.

	aDescriptor table: userTable.
	
	(aDescriptor newMapping: DirectMapping) from: #id to: (userTable fieldNamed: 'ID').
	(aDescriptor newMapping: DirectMapping) from: #username to: (userTable fieldNamed: 'USERNAME').
	(aDescriptor newMapping: DirectMapping) from: #passwordHash to: (userTable fieldNamed: 'PASSWORDHASH').

	(aDescriptor newMapping: ManyToManyMapping)
		attributeName: #roles;
		referenceClass: ITRole;
		useLinkTable;
		mappingCriteria: (Join
			from: (userTable fieldNamed: 'ID')
			to:  (userRoleTable fieldNamed: 'USER_ID')
		).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 20:47'!
tableForCOMMENT: aTable
	"COMMENTS(ID, ISSUE_ID, TIMESTAMP, CONTENT)
	          --"
	
	| issueId userId|
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueId := aTable createFieldNamed: 'ISSUE_ID' type: (platform int4).
	userId := aTable createFieldNamed: 'USER_ID' type: (platform int4).	
	aTable createFieldNamed: 'TIMESTAMP' type: (platform timestamp).
	aTable createFieldNamed: 'CONTENT' type: (platform text).
	
	"foreign key definition for the IDs of issues"
	aTable 
		addForeignKeyFrom: issueId 
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
	aTable 
		addForeignKeyFrom: userId 
		to: ((self tableNamed: 'USERS') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 20:47'!
tableForDETAIL: aTable
	"DETAIL(ID, ISSUE_ID, TAG, CONTENT)
	       --"
	
	| issueID |
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.
	issueID := (aTable createFieldNamed: 'ISSUE_ID' type: (platform int4)).
	aTable createFieldNamed: 'TAG' type: (platform varchar: 255).
	aTable createFieldNamed: 'CONTENT' type: (platform varchar: 255).
	
	"foreign key definition for IDs of issues"
	aTable
		addForeignKeyFrom: issueID
		to: ((self tableNamed: 'ISSUE') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 12/14/2007 09:38'!
tableForISSUE: aTable
	"ISSUE(ID)
	       --"
	
	(aTable createFieldNamed: 'ID' type: (platform sequence)) bePrimaryKey.! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 00:50'!
tableForROLE: aTable
	"ROLE(ID, ROLENAME)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'ROLENAME' type: (platform varchar: 255).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 21:36'!
tableForROLEPRIVILEGE: aTable
	"ROLEPRIVILEGE(ROLE_ID, PRIVILEGE, PRIVILEGE_ORDER)"
	
	| roleid privilege |
	
	roleid := aTable createFieldNamed: 'ROLE_ID' type: platform int4.
	privilege := aTable createFieldNamed: 'PRIVILEGE' type: platform text.
	aTable createFieldNamed: 'PRIVILEGE_ORDER' type: platform int4.
	
	roleid bePrimaryKey.
	privilege bePrimaryKey.
	
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		suffixExpression: 'ON DELETE CASCADE'.
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'afk 1/6/2008 12:29'!
tableForTAGS: aTable
	"USERS(ID, USERNAME, PASSWORDHASH)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'TAGNAME' type: (platform varchar: 255).
	aTable createFieldNamed: 'TAGDESCR' type: (platform varchar: 255).! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mt 1/7/2008 20:47'!
tableForUSERROLE: aTable
	"USERROLE(USER_ID, ROLE_ID)"

	| userid roleid |
	
	userid := aTable createFieldNamed: 'USER_ID' type: (platform int4).
	roleid := aTable createFieldNamed: 'ROLE_ID' type: (platform int4).
		
	aTable 
		addForeignKeyFrom: userid 
		to: ((self tableNamed: 'USERS') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
		
	aTable 
		addForeignKeyFrom: roleid 
		to: ((self tableNamed: 'ROLE') fieldNamed: 'ID')
		"suffixExpression: 'ON DELETE CASCADE'".
! !

!ITGlorpDescriptorSystem methodsFor: 'tables' stamp: 'mig 12/17/2007 01:02'!
tableForUSERS: aTable
	"USERS(ID, USERNAME, PASSWORDHASH)"
	
	(aTable createFieldNamed: 'ID' type: platform sequence) bePrimaryKey.
	aTable createFieldNamed: 'USERNAME' type: (platform varchar: 255).
	aTable createFieldNamed: 'PASSWORDHASH' type: (platform varchar: 255).! !

Object subclass: #ITGlorpDB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITGlorpDB commentStamp: 'mt 12/14/2007 10:22' prior: 0!
I encapsulate the connection to a SQL database, at the moment with PostGreSQL. Feel free to use my class side method #connect to get a new ITGlorpSession instance. But be aware of the connection. You'll have to check whether ITGlorpSession>>#isLoggedIn.!

!ITGlorpDB class methodsFor: 'debug' stamp: 'mt 12/14/2007 10:37'!
closeAllConnections
	ITGlorpSession allInstances do: [:aSession |
		aSession logout.
		Transcript show: 'DEBUG: You closed a Session.']! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 12/15/2007 12:30'!
connect
	"Connects to the database with the given information from login and returns the current (new) session."	
	
	| accessor session system |
	accessor := DatabaseAccessor forLogin: self login.
	accessor login.

	session := GlorpSession new.
	session accessor: accessor.

	system := ITGlorpDescriptorSystem new.
	session system: system.
	system platform: session platform.
	
	^ session
! !

!ITGlorpDB class methodsFor: 'debug' stamp: 'mig 1/7/2008 12:12'!
createTables
	"I'm creating alle relevant tables but only if there are none of them."
	
	| session |
	Transcript show: 'DEBUG: You try to create all tables again.'.
	session := self connect.
	session inTransactionDo: [session system allTables do: [:each |
			Transcript cr; show: each name; cr.
			session accessor createTable: each ifError: [:error | Transcript show: error messageText]
		]
	].

	session inTransactionDo: [session system allTables do: [:each |
	session accessor createTableFKConstraints: each ifError: [:error | Transcript show: error messageText]		
		]
	].! !

!ITGlorpDB class methodsFor: 'connection' stamp: 'mt 1/8/2008 21:35'!
login
	"The default login can be found here. It is used across the whole issue tracker application. It is also used for unit tests."
	
	^ Login new database: PostgreSQLPlatform new;
		username: 'issuetracker';
		password: 'weloveseasidedontwe';
		connectString: 'theresnoplacelike.homeftp.org:5432_issuetracker'.
		
	"^ Login new database: PostgreSQLPlatform new;
		username: 'postgres';
		password: '1234';
		connectString: 'localhost:5432_postgres'."! !

Object subclass: #ITIssue
	instanceVariableNames: 'id details comments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssue commentStamp: 'mt 12/30/2007 14:33' prior: 0!
I am an issue, a task, a ticket ... or however you'll call me.

I encapsulate several details and comments.!

!ITIssue class methodsFor: 'instance creation' stamp: 'afk 12/13/2007 20:12'!
with: aTitle with: aComment

	| newIssue comment |
	newIssue := self new.
	comment := ITIssueComment new.
	
	comment content: aComment.
	
	newIssue comments add: comment.
	newIssue details add: (ITIssueDetail with: 'title' with: aTitle).

	^ newIssue! !

!ITIssue methodsFor: 'changing' stamp: 'mt 1/5/2008 11:23'!
addComment: aComment

	self comments add: aComment."(ITIssueComment with: aString)."! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 11/23/2007 19:55'!
comments
	^comments.! !

!ITIssue methodsFor: 'accessing' stamp: 'mt 12/11/2007 15:23'!
comments: aCollection
	comments := aCollection.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details
	^ details! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 17:03'!
details: anObject
	details := anObject! !

!ITIssue methodsFor: 'comments' stamp: 'mt 1/6/2008 14:13'!
firstComment
	"Returns the first added comment when the issue was created."
	
	^self comments first.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 12/13/2007 18:18'!
getTag: tag
	details do: 
		[:each | 
		each tag = tag
			ifTrue: [^ each content]
		].
	^nil.! !

!ITIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 16:20'!
id
	^ id! !

!ITIssue methodsFor: 'changing' stamp: 'afk 11/12/2007 16:20'!
id: anObject
	id := anObject! !

!ITIssue methodsFor: 'initialize-release' stamp: 'mt 12/15/2007 16:20'!
initialize

	super initialize.
	details := OrderedCollection new.
	comments := OrderedCollection new.

	! !

!ITIssue methodsFor: 'comments' stamp: 'mt 1/6/2008 14:13'!
latestComment
	"Returns the last added comment."
	
	^self comments last.! !

!ITIssue methodsFor: 'changing' stamp: 'mt 12/15/2007 16:20'!
setDetailWith: aTag to: aValue
	self details add: (ITIssueDetail with: aTag with: aValue)! !

Object subclass: #ITIssueComment
	instanceVariableNames: 'id content user timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueComment commentStamp: 'mt 12/30/2007 14:30' prior: 0!
I am just an annotation to an issue.

You should be able to see which user created me, what's the content and when that user created me.!

!ITIssueComment class methodsFor: 'instance creation' stamp: 'mt 1/8/2008 12:21'!
with: aString
	self flag: #OBSOLETE.
	^ self new
		content: aString;
		yourself.! !

!ITIssueComment class methodsFor: 'instance creation' stamp: 'mt 1/5/2008 11:37'!
with: aString by: aUser
	^ self new
		content: aString;
		user: aUser;
		timestamp: TimeStamp current;
		yourself.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content
	^ content! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
content: anObject
	content := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id
	^ id! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 17:18'!
id: anObject
	id := anObject! !

!ITIssueComment methodsFor: 'initialize-release' stamp: 'mt 12/14/2007 10:02'!
initialize
	
	super initialize.
	
	content := ''.! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/8/2008 17:02'!
printTimeStamp
	"Returns a formatted string of the timestamp."
	
	^(self timestamp date printRelativeDate, ', ', self timestamp time print24).! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:53'!
timestamp
	^ timestamp! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:53'!
timestamp: anObject
	timestamp := anObject! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:56'!
user
	^ user! !

!ITIssueComment methodsFor: 'accessing' stamp: 'mt 1/5/2008 10:56'!
user: anObject
	user := anObject! !

Object subclass: #ITIssueDetail
	instanceVariableNames: 'id tag content user timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueDetail commentStamp: 'mt 12/30/2007 14:28' prior: 0!
I represent an issue detail.

What's a detail? Well... it's a tag-value-combination. You have some tag like 'title' and the proper value like 'This is the title'. These two things form a 'detail'. There are other well-definied tags out there but you should be able to create your own one in the application (e.g. as an administrator).!

!ITIssueDetail class methodsFor: 'instance creation' stamp: 'mig 12/20/2007 18:51'!
with: aTag with: aValue
	
	| detail |
	detail := self new.
	detail tag: aTag.
	detail content: aValue.
	
	^ detail.! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content
	^ content! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:54'!
content: anObject
	content := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id
	^ id! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
id: anObject
	id := anObject! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag
	^ tag! !

!ITIssueDetail methodsFor: 'accessing' stamp: 'afk 12/13/2007 16:52'!
tag: anObject
	tag := anObject! !

Object subclass: #ITIssueManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITIssueManager commentStamp: 'mt 12/30/2007 14:22' prior: 0!
I am responsible for any "issue-related" operations.

Every issue operation has to go through my methods. I determine which privilege is needed to perfom several actions. 

DO NOT talk to the ITSecurityManager about security-related things assigned to issues in any ITComponent - I will do that for you.!

!ITIssueManager methodsFor: 'accessing' stamp: 'mt 1/5/2008 11:41'!
addComment: aString atID: aNumber by: aUser
	"Adds a new ITIssueComment object to the issue with the ID 'aNumber'."

	| issue glorpSession comment |
	glorpSession := ITGlorpSession getInstance.
	issue := glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].

	comment := ITIssueComment with: aString by: aUser.
	
	glorpSession inUnitOfWorkDo: [
		glorpSession register: issue.
		glorpSession register: comment.
		issue addComment: comment.
	].! !

!ITIssueManager methodsFor: 'changing' stamp: 'mig 1/8/2008 14:12'!
addIssueWithTitle: aString
	"Adds a new issue with a detail 'title' to the database. Returns this new issue."
	
	| session issue |
	
	ITSecurityManager checkPrivilege: #'add_issue'.
	
	session := ITGlorpSession getInstance.
	
	issue := ITIssue new.
	issue setDetailWith: 'title' to: aString.
	
	session inUnitOfWorkDo: [
		session register: issue.
	].

	^issue.! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 1/8/2008 14:11'!
deleteIssueWithID: aNumber
	"Tries to delete an issue with the given ID from the database."
	
	| session issue |
	
	ITSecurityManager checkPrivilege: #'delete_issue'.
	
	session := ITGlorpSession getInstance.
	
	issue := session readOneOf: ITIssue where: [:each | each id = aNumber].
	
	session delete: issue.
	! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 1/8/2008 14:11'!
getIssueWithID: aNumber
	"Returns an ITIssue object where the 'id' equals 'aNumber'."
	| glorpSession |
	
	ITSecurityManager checkPrivilege: #'read_issue'.

	glorpSession := ITGlorpSession getInstance.
	^ glorpSession readOneOf: ITIssue where: [:each | each id = aNumber].
! !

!ITIssueManager methodsFor: 'accessing' stamp: 'mig 1/8/2008 14:12'!
getIssues
	"Returns all ITIssue objects in a collection that are in the database"
	
	ITSecurityManager checkPrivilege: #'list_issues'.

	^ ITGlorpSession getInstance readManyOf: ITIssue.
	! !

Object subclass: #ITRole
	instanceVariableNames: 'id rolename privileges users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITRole commentStamp: 'mt 12/30/2007 14:37' prior: 0!
I am a user role and encapsulate a group of privileges.!

!ITRole methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:07'!
addPrivilege: aSymbol
	privileges add: aSymbol asString! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/7/2008 22:51'!
addUser: aUser
	users add: aUser.! !

!ITRole methodsFor: 'testing' stamp: 'mt 12/30/2007 11:24'!
hasPrivilege: aSymbol
	^ privileges includes: aSymbol! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id
	^ id! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:23'!
id: anObject
	id := anObject! !

!ITRole methodsFor: 'initialization' stamp: 'mt 1/5/2008 23:31'!
initialize
	super initialize.
	
	privileges := Set new.
	users := Set new.
	rolename := 'unnamed'.! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:49'!
privileges
	^ privileges! !

!ITRole methodsFor: 'accessing' stamp: 'mt 12/30/2007 11:24'!
privileges: somePrivileges
	"Ensure that privileges are a set."
	
	privileges := somePrivileges asSet! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:09'!
removePrivilege: aSymbol
	privileges remove: aSymbol asString.! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/7/2008 23:13'!
removeUser: aUser
	"Tries to remove a user from the role and does nothing if that user is absent."

	users remove: aUser ifAbsent: [].! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename
	^ rolename! !

!ITRole methodsFor: 'accessing' stamp: 'mig 12/16/2007 15:53'!
rolename: anObject
	rolename := anObject! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/5/2008 23:27'!
users
	^ users! !

!ITRole methodsFor: 'accessing' stamp: 'mt 1/7/2008 22:47'!
users: someUsers
	"Ensure that someUsers are a set."
	
	users := someUsers asSet.! !

Object subclass: #ITSecurityManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSecurityManager commentStamp: 'mt 12/30/2007 14:23' prior: 0!
I am responsible for application security operations.

You ask me to add and retrieve users and roles from/to the database. I'm able to hash a password which is in plaintext - that is more secure than saving them without.

Please DO NOT let me add any issue related stuff to the database - that's the job of the ITIssueManager. Although I'm able to verify passwords to usernames and given privileges for a user, it's also the job of the ITIssueManager to assign safety critical operations on issues to needed privileges.!

!ITSecurityManager class methodsFor: 'testing' stamp: 'afk 1/8/2008 12:58'!
checkPrivilege: aSymbol 
	"check whether the current user has the 'aSymbol'-privilege; always allow if no current session (testing)"
	| currentSession |
	
	self flag:#TODO. "Using of WACurrentSession discouraged? Move user to SM?"
	
	WACurrentSession value ifNil: [^true].
	
	currentSession := WACurrentSession value.
	
	(currentSession securityManager hasUser: currentSession user privilege: aSymbol) ifFalse: [
		ITUnauthorizedError signal.
	].

	^ true.! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashAlgorithm
	^MD5! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:13'!
hashPassword: aString
	^ (self hashAlgorithm hashMessage: aString) hex.
	! !

!ITSecurityManager class methodsFor: 'hashing' stamp: 'mig 12/22/2007 17:15'!
hashSize
	"hash size is algorithms size * 2 because hashs are stored as hex"
	^ self hashAlgorithm hashSize * 2
	! !

!ITSecurityManager class methodsFor: 'testing' stamp: 'afk 1/9/2008 16:46'!
privileges
	"Returns all available privileges that are used by the issue manager or the security manager. Please note that privileges that are not listed here cannot be attached to a role with the web frontend."

	^#(#'read_issue' #'add_issue' #'delete_issue' #'list_issues' #'administrate')! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mig 1/9/2008 15:18'!
addRole: aRole
	"add an ITRole object to the database"
	| session |
	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aRole.
	].

	^aRole! !

!ITSecurityManager methodsFor: 'traits' stamp: 'mig 1/9/2008 16:16'!
addUser: aUser
	"add an ITUser object to the database, but only if there is not already a user with the given name present"
	| session |
	
	(self getUser: aUser username) ifNotNil: [
		ITUserAlreadyExists signal.
	].
	
	session := ITGlorpSession getInstance.
	
	session inUnitOfWorkDo: [
		session register: aUser.
	].

	^aUser! !

!ITSecurityManager methodsFor: 'defaults' stamp: 'mig 1/9/2008 15:19'!
defaultUser
	"return the default user (the unauthenticated user); create default user if it does not exist"
	
	^ (self getUser: 'Guest') ifNil: [
		self addUser: (ITUser new username: 'Guest')
	].! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 1/5/2008 23:47'!
deleteRoleWithID: aNumber
	"Removes a role with the given ID from the database."
	
	|session role|
	session := ITGlorpSession getInstance.
	
	role := session readOneOf: ITRole where: [:each | each id = aNumber].
	
	session delete: role.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 12/30/2007 12:10'!
deleteUserWithID: aNumber
	"Removes a user with the given ID from the database."
	
	|session user|
	session := ITGlorpSession getInstance.
	
	user := session readOneOf: ITUser where: [:each | each id = aNumber].
	
	session delete: user.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:22'!
getRole: aString
	"get ITRole object from database"
	| session |
	
	session := ITGlorpSession getInstance.
	^session readOneOf: ITRole where: [:each | each rolename = aString].
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 12/30/2007 12:13'!
getRoleWithID: aNumber
	"get ITRole object with the given ID from database"
	
	| session |
	session := ITGlorpSession getInstance.
	^session readOneOf: ITRole where: [:each | each id = aNumber]
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 1/9/2008 16:43'!
getUser: aString
	"get ITUser object from database where username is aString (case-insensitive)"
	| session |
	
	session := ITGlorpSession getInstance.
	^session readOneOf: ITUser where: [:each | each username asLowercase = aString asLowercase].
	! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mt 12/30/2007 12:12'!
getUserWithID: aNumber
	"get ITUser object with the given ID from database"
	
	| session |
	session := ITGlorpSession getInstance.
	^session readOneOf: ITUser where: [:each | each id = aNumber]
	! !

!ITSecurityManager methodsFor: 'testing' stamp: 'mig 1/9/2008 15:13'!
hasUser: aUser privilege: aSymbol
	"Checks whether aUser has the aSymbol privilege."
	
	aUser roles do: [:role |
		(role hasPrivilege: aSymbol) ifTrue: [^true].
	].

	^false.! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'afk 1/8/2008 13:01'!
loginUser: aUserName withPassword: aUserPassword
	"Logic for logging in a user"

	| valid currentSession |
	
	self flag:#TODO. "This is probably not god. Move the user to SM ?"
	WACurrentSession value ifNil: [^false].
	currentSession := WACurrentSession value.
	
	valid := self validatePassword: aUserPassword for: aUserName.
	valid ifTrue: [
			currentSession user: (self getUser: aUserName).
			^ true
	] ifFalse: [
			^ false]! !

!ITSecurityManager methodsFor: 'as yet unclassified' stamp: 'mig 12/22/2007 17:21'!
validatePassword: aPassword for: aUsername
	"return whether aUsername and aPassword match"
	| user pwhash |
	
	user := self getUser: aUsername.
	pwhash := self class hashPassword: aPassword.
	
	user ifNil: [^false].
	^ user passwordHash = pwhash.! !

Object subclass: #ITTag
	instanceVariableNames: 'id name description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITTag class methodsFor: 'instance creation' stamp: 'afk 1/9/2008 19:03'!
with: aTagName with: aTagDescription
	"Creates new Tag that later can be used by the user"

	| newTag |
	newTag := self new.
	
	newTag name: aTagName.
	newTag description: aTagDescription.

	^ newTag! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 19:02'!
description
	^ description! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 19:02'!
description: anObject 
	description := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:22'!
id
	^ id! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/6/2008 12:22'!
id: anObject
	id := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 18:57'!
name
	^ name! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 18:57'!
name: anObject 
	name := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 19:01'!
tagDescription
	^ description! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 19:01'!
tagDescription: anObject 
	description := anObject! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 18:53'!
tagName
	^ name! !

!ITTag methodsFor: 'accessing' stamp: 'afk 1/9/2008 18:53'!
tagName: anObject 
	name := anObject! !

Object subclass: #ITTranslator
	instanceVariableNames: 'phraseBooks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

ITTranslator class
	instanceVariableNames: 'instance'!

!ITTranslator commentStamp: 'mt 12/26/2007 10:42' prior: 0!
I am responsible for managing all localization/internationalization issues in this application. You have to load a valid translation file and then any ByteString can be translated using ByteString>>#tr.!

ITTranslator class
	instanceVariableNames: 'instance'!

!ITTranslator class methodsFor: 'as yet unclassified' stamp: 'mig 1/8/2008 12:36'!
findTexts
	"return all literals with tr message (i.e. find all texts to translate)"
	
	| texts pts |
	pts := ParseTreeSearcher new.
	texts := Set new.
	
	pts matches: '`#literal tr' do: [:aNode :answer | texts add: aNode children first value].

	(SystemOrganization superclassOrder: 'SWE-IssueTracker') do: [:aClass |
		aClass selectorsDo: [:aSelector |
			pts executeTree: (aClass parseTreeFor: aSelector).
		].
	].

	^ texts.! !

!ITTranslator class methodsFor: 'accesssing' stamp: 'mt 12/26/2007 10:53'!
getInstance
	"Using singleton pattern to ensure that there is only one instance of ITTranslator"
	
	instance ifNil:
		[ instance := self basicNew.].
	^ instance! !

!ITTranslator class methodsFor: 'instance creation' stamp: 'mt 12/26/2007 10:50'!
new
	Error signal: 'Use getInstance because ', self name, ' is a singleton'.! !

!ITTranslator methodsFor: 'translation' stamp: 'mt 12/26/2007 12:19'!
discardTranslationsForLanguage: aSymbol
	"Discards the given phrase book."
	
	phraseBooks
		removeKey: aSymbol
		ifAbsent: ["No such phrase book found."].! !

!ITTranslator methodsFor: 'initialization' stamp: 'mt 12/26/2007 11:49'!
initialize
	super initialize.
	
	phraseBooks := Dictionary new.! !

!ITTranslator methodsFor: 'translation' stamp: 'mt 12/26/2007 12:12'!
loadTranslationFile: aFilePath forLanguage: aSymbol
	"Creats a new phrase book for lookup."
	
	self flag: #TODO
! !

!ITTranslator methodsFor: 'private' stamp: 'mt 12/26/2007 11:50'!
phraseBookFor: aLanguage
	^phraseBooks at: aLanguage.! !

!ITTranslator methodsFor: 'translation' stamp: 'mt 12/26/2007 12:23'!
translate: aByteString into: aLanguage

	|phraseBook translation|	
	[phraseBook := self phraseBookFor: aLanguage]
		on: Error "key not found"
		do: [translation := aByteString].
	
	[translation := phraseBook at: aByteString]
		on: Error "key not found or invalid phrase book"
		do: [translation := aByteString].
	
	^translation.! !

Object subclass: #ITUser
	instanceVariableNames: 'id username passwordHash roles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUser commentStamp: 'mt 12/30/2007 14:36' prior: 0!
I represent a user which is utilizing this application.

I have a username, a password and several roles that are attached to me. These roles enable me to do some operations through the ITIssueManager on the database.!

!ITUser methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:00'!
addRole: aRole
	roles add: aRole.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id
	^ id! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/16/2007 16:26'!
id: anObject
	id := anObject! !

!ITUser methodsFor: 'initialization' stamp: 'mt 12/29/2007 09:49'!
initialize
	super initialize.
	
	roles := Set new.! !

!ITUser methodsFor: 'testing' stamp: 'mt 12/30/2007 11:08'!
isInRole: aRole
	^roles includes: aRole! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
passwordHash
	^ passwordHash! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/17/2007 15:10'!
removeRole: aRole
	roles remove: aRole.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
roles
	^ roles! !

!ITUser methodsFor: 'accessing' stamp: 'mt 12/30/2007 11:21'!
roles: someRoles
	"Ensure that roles are a set."

	roles := someRoles asSet! !

!ITUser methodsFor: 'passwords' stamp: 'mig 12/17/2007 13:36'!
setPassword: aString
	passwordHash := ITSecurityManager hashPassword: aString.! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username
	^ username! !

!ITUser methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:27'!
username: anObject
	username := anObject! !

WASession subclass: #ITSession
	instanceVariableNames: 'issueManager language securityManager user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITSession commentStamp: 'mt 12/30/2007 14:42' prior: 0!
I am a normal Seaside session. You can get to your personal security manager and issue manager through my methods.

I have exactly one current language and one active user.!

!ITSession methodsFor: 'testing' stamp: 'mig 1/9/2008 15:39'!
isLoggedIn
	"A default user is NOT logged in."	

	^self user ~= self securityManager defaultUser! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/30/2007 15:49'!
issueManager
	^ issueManager ifNil: [ issueManager := ITIssueManager new ].
! !

!ITSession methodsFor: 'accessing' stamp: 'mt 12/26/2007 11:36'!
language
	^ language! !

!ITSession methodsFor: 'accessing' stamp: 'mt 12/26/2007 11:36'!
language: anObject
	language := anObject! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/30/2007 15:49'!
securityManager
	^ securityManager ifNil: [ securityManager := ITSecurityManager new ].
! !

!ITSession methodsFor: 'accessing' stamp: 'mig 1/9/2008 15:28'!
user
	"Returns the current user object or the default user if nobody is logged in. NIL is never returned."
	
	^user ifNil: [ user := self securityManager defaultUser ]
! !

!ITSession methodsFor: 'accessing' stamp: 'mig 12/10/2007 17:46'!
user: anObject
	user := anObject! !

!ByteString methodsFor: '*SWE-IssueTracker' stamp: 'mt 12/26/2007 11:42'!
tr
	"Uses ITTranslator for translating this string into another language."	
		
	|lang|
	[lang := WACurrentSession language]
		on: MessageNotUnderstood "for testing purpose only"
		do: [lang := #default].
		
	^ITTranslator getInstance translate: self into: lang.! !

!Date methodsFor: '*SWE-IssueTracker' stamp: 'mt 1/8/2008 17:02'!
printRelativeDate
	"Returns 'Today' or 'Yesterday' or 'MM-DD-YYYY' as String for the date."
	
	self = Date today ifTrue: [^'Today' tr].
	self = Date yesterday ifTrue: [^'Yesterday' tr].
	
	^(self printFormat: #(1 2 3 $- 1 1 2))! !

WAComponent subclass: #ITComponent
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponent commentStamp: 'mig 1/9/2008 11:56' prior: 0!
I am the base class for all components (ITComponent*) in this project.!

!ITComponent methodsFor: 'user interface' stamp: 'mig 1/9/2008 17:37'!
inform: aString
	^self inform: aString title: 'Information' tr! !

!ITComponent methodsFor: 'user interface' stamp: 'mig 1/9/2008 17:37'!
inform: message title: title 
	
	^self inform: message title: title buttons: {'OK' tr}.! !

!ITComponent methodsFor: 'user interface' stamp: 'mig 1/9/2008 17:35'!
inform: message title: title buttons: buttons
	"show a message (light-)box with title, message and buttons"
	
	^self lightbox: (ITComponentMessage new title: title; message: message; buttons: buttons; yourself).! !

!ITComponent methodsFor: 'accessing' stamp: 'mfp 1/8/2008 16:08'!
parent

	^ parent.! !

!ITComponent methodsFor: 'accessing' stamp: 'mfp 1/8/2008 16:07'!
parent: anObject 

	parent = anObject.! !

!ITComponent methodsFor: 'rendering' stamp: 'mig 1/8/2008 16:18'!
renderButtonOn: html with: text action: anUpdate

	self flag: #OBSOLETE.

	html anchor
		class: 'button';
		onClick: anUpdate;
		with: [ html span
			class: 'button';
			with: [html text: text]
		]! !

ITComponent subclass: #ITComponentAddComment
	instanceVariableNames: 'comment currentIssue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddComment commentStamp: 'mig 1/9/2008 11:59' prior: 0!
I am responsible to provide an interface for adding comments to an issue.!

!ITComponentAddComment class methodsFor: 'instance creation' stamp: 'mt 1/4/2008 23:17'!
newWith: anID
	| newInstance |
	newInstance := self new.
	newInstance currentIssue: (newInstance session issueManager getIssueWithID: anID).
	^newInstance.! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment
	^ comment! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mig 11/30/2007 19:11'!
comment: anObject
	comment := anObject! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:16'!
currentIssue
	^ currentIssue! !

!ITComponentAddComment methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:16'!
currentIssue: anObject
	currentIssue := anObject! !

!ITComponentAddComment methodsFor: 'rendering' stamp: 'mt 1/5/2008 11:26'!
renderContentOn: html

	html div: [
		html anchor
			onClick: (html effect id: 'new_comment_form'; perform: #toggleBlind);
			with: 'Add Comment' tr.
				
		html form
			id: 'new_comment_form';
			style: 'display: none;';
			with: [
				html textArea on: #comment of: self.
				html break.
				html submitButton
					callback: [ 
						self session issueManager
							addComment: self comment
							atID: self currentIssue id
							by: self session user
					];
					text: 'Add Comment' tr.
			]
	]! !

ITComponent subclass: #ITComponentAddIssue
	instanceVariableNames: 'title newTags newTagName initialComment newTagValue tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAddIssue commentStamp: 'mig 1/9/2008 11:59' prior: 0!
I am responsible to provide an interface for creating new issues.!

!ITComponentAddIssue methodsFor: 'private' stamp: 'mig 1/7/2008 12:21'!
deleteEntries
	"Delete data in inputfields after clicking cancel or submit"

	self 
		initialComment: nil;
		title: nil;
		newTagValue: nil;
		newTagName: nil;
		newTags: Dictionary new.
	! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'afk 1/7/2008 00:57'!
deleteTagEntries
	self newTagName: nil.
	self newTagValue: nil.! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment
	^ initialComment! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
initialComment: anObject
	initialComment := anObject! !

!ITComponentAddIssue methodsFor: 'initialize-release' stamp: 'afk 1/7/2008 00:57'!
initialize
	
	super initialize.
	self newTags: (Dictionary new).
	self newTagName: nil.! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:38'!
newTagName
	^ newTagName! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:38'!
newTagName: anObject
	newTagName := anObject! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/5/2008 20:32'!
newTagValue
	^ newTagValue! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/5/2008 20:32'!
newTagValue: anObject
	newTagValue := anObject! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/5/2008 21:37'!
newTags
	^ newTags! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/5/2008 21:37'!
newTags: anObject
	newTags := anObject! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'afk 1/9/2008 18:58'!
renderChooseTagOn: html
	"User can choose what tags he wants to add and what value the tag should have "

	self tags: (ITGlorpSession getInstance readManyOf: ITTag).

	html select
		list: (self tags asOrderedCollection collect: [:each | each name ]);
		selected: self newTagName;
		callback: [ :value | self newTagName: value ];
			onChange: (html updater 
				id: 'tag_description';
				"triggerForm: 'tag_selection';"
				callback: [:r | r text: 'Description: ' tr, self newTagName tr.]
				).
	! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'afk 1/9/2008 17:48'!
renderContentOn: html
	
	[ITSecurityManager checkPrivilege: #'add_issue'.]
		on: ITUnauthorizedError
		do: [html text: 'You are not authorized to add issues in this awesome application!!' tr. ^nil].
	
	html div
		class: 'content';
		with: [
			html heading: 'Add Issue' tr level: 1.	
			html form: 
				[
				html text: 'Title' tr.
					html break.
				html textInput on: #title of: self.
					html break.
					html break.
				self renderCurrentTagsOn: html.
					html break.
				self renderTagFormOn: html.
					html break. 
					html break.
				html text: 'Description' tr.
					html break.
				html textArea on: #initialComment of: self.
					html break.
			
				html submitButton 
					callback: [
						self submit.		
						self deleteEntries.	
						self answer: true. 
					];
					value: 'add Issue' tr.
				html cancelButton
					callback: [
						self deleteEntries.
						self answer: false];
					value: 'cancel' tr.
		]]! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'afk 1/7/2008 00:55'!
renderCreateNewTagOn: html
	"Renders the possibility to add new tags while creating a new Issue"

	html fieldSet
		legend: 'New tag:' tr;
		id: 'new_tag_fieldset';
		style: '	font-weight: normal;
				color: black;';	
		with: [
			html form
				id: 'new_tag_form';
				with: 	[
					html text: 'Name' tr.
						html space.
					self renderChooseTagOn: html.				
						html space.
					html text: 'Description: '.
					html text: 'The description of the tag. TODO'.
						html break.
					html text: 'Value' tr.
						html space.
					html textInput on: #newTagValue of: self.
						html space.
					html submitButton 
						callback: [	
							self newTagName ifNotNil: [self newTags at: self newTagName put: self newTagValue].
							self deleteTagEntries.
							self answer: true. 
								];
						value: 'add this Tag' tr.
					html break.
						].
		].! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'afk 1/6/2008 16:37'!
renderCurrentTagsOn: html
	"Shows the tags that the user provided to be added. User has the abillity to remove them before saving"

	html fieldSet
		legend: 'Current tags:' tr;
		style: '	font-weight: normal;
				color: black;';
		with: [ 	self newTags associationsDo: [:assoc | 
				html div
					id: (assoc key);
					with: [
						html strong: 'Tag: ' tr.
						html text: (assoc key). 
						html space. 
						html strong: 'Content: ' tr.
						html text: (assoc value). 
						html space.
																							
						html anchor
							onClick: (html updater
								id: (assoc key);
								callback: [:render | self newTags removeKey: (assoc key).]);
							with: 'Delete'.
												
						html break.]
											]
				].! !

!ITComponentAddIssue methodsFor: 'rendering' stamp: 'mig 1/8/2008 17:40'!
renderTagFormOn: html
	"Renders anchor for the form to add a new tag "

	html div
		id: 'tag_adding';
		with: [].	
	
	html anchor
		onClick: (html updater
			id: 'tag_adding';
			callback: [ :render | self renderCreateNewTagOn: render]);
		accessKey: $t;
		with: 'Write another Tag' tr.! !

!ITComponentAddIssue methodsFor: 'private' stamp: 'mt 1/7/2008 23:47'!
submit
	"Submits the data an creates a new issue with an initial comment."
	
	|issue|
	[issue := self session issueManager addIssueWithTitle: self title]
		on: ITUnauthorizedError 
		do: [self inform: 'You are not authorized to add an issue!!' tr. ^nil].
	
	self newTags associationsDo: [:assoc | issue setDetailWith: (assoc key) to: (assoc value) ].
	
	[self session issueManager addComment: self initialComment atID: issue id by: self session user]
		on: ITUnauthorizedError 
		do: [self inform: 'You are not authorized to add a comment to an issue!!' tr. ^nil].
! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:43'!
tags
	^ tags! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 1/6/2008 23:43'!
tags: anObject
	tags := anObject! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title
	^ title! !

!ITComponentAddIssue methodsFor: 'accessing' stamp: 'afk 11/12/2007 15:49'!
title: anObject
	title := anObject! !

ITComponent subclass: #ITComponentAdmin
	instanceVariableNames: 'userTabs tagTabs tabsRoot selectedTabKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAdmin commentStamp: 'mt 1/6/2008 18:28' prior: 0!
I am rendering the frontend for the administration area. Here you can manage custom tags and attach roles to users. You can also create new roles and modify the privileges each role has.

If you want to add more (sub-)sections just modifiy #refreshTabs.!

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/6/2008 17:51'!
children
	^Array with: self currentChild.! !

!ITComponentAdmin methodsFor: 'private' stamp: 'mt 1/6/2008 17:50'!
currentChild
	"Returns the component which is the current child - otherwise return nil."
	
	self tabsRoot valuesDo: [:subsections |
		(subsections includesKey: self selectedTabKey)
			ifTrue: [^(subsections at: self selectedTabKey)]
		].
	^nil! !

!ITComponentAdmin methodsFor: 'initialization' stamp: 'mt 1/6/2008 18:21'!
initialize
	"Creates all tabs once but does not select one to show the introduction for this component to the user. See #renderIntroductionOn: for more details."
	
	super initialize.
	self refreshTabs.
	self selectedTabKey: ''.! !

!ITComponentAdmin methodsFor: 'private' stamp: 'mt 1/6/2008 18:01'!
refreshTabs
	"Creates new component objects for the children but keeps the current selected tab."
	
	self userTabs: (Dictionary new
		at: 'User -> Roles' tr put: ITComponentAdminUsers new;
		at: 'Roles -> Privileges' tr put: ITComponentAdminRoles new;
		yourself).

	self tagTabs: (Dictionary new
		at: 'Tags' tr put: ITComponentAdminDetails new;
		yourself).

	self tabsRoot: (Dictionary new
		at: 'User Control' tr put: self userTabs;
		at: 'Custom Tags' tr put: self tagTabs;
		yourself).
	
				
	! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/6/2008 18:09'!
renderChildrenOn: html
	html div
		class: 'content_with_menu';
		with: [
			self selectedTabKey = ''
				ifTrue:  [ self renderIntroductionOn: html ]
				ifFalse: [ html render: self currentChild ]
		]! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'afk 1/9/2008 16:56'!
renderContentOn: html

	[ITSecurityManager checkPrivilege: #'administrate'.]
		on: ITUnauthorizedError
		do: [html text: 'You are not authorized to administrate this awesome application!!' tr. ^nil].

	html div
		with: [
			self renderSideMenuOn: html.
			self renderChildrenOn: html.
		]! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/6/2008 18:19'!
renderIntroductionOn: html
	"This shows a brief introduction to the admin area."
	
	html heading
		level: 2;
		with: 'Welcome to the Administration Area!!' tr.
		
	html paragraph	with: 'In this area you can modify several security settings:' tr.
	html unorderedList with: [
		html listItem with: 'attach roles to users' tr.
		html listItem with: 'detach roles from users' tr.
		html listItem with: 'create new roles' tr.
		html listItem with: 'modify privileges for each role' tr.
		html listItem with: 'delete roles' tr.].
	html paragraph with: 'Additionally, you can create your own new tags. These custom tags can be added to any issue by all users which are allowed to add or modify them. Every custom tag contains:' tr.
	html unorderedList with: [
		html listItem with: 'a descriptive title' tr.
		html listItem with: 'a clear and brief description of the tag' tr.
		html listItem with: 'the required input field with default values (not yet)' tr].
	html paragraph with: 'Please apply any changes with caution!!' tr.
			! !

!ITComponentAdmin methodsFor: 'rendering' stamp: 'mt 1/6/2008 17:36'!
renderSideMenuOn: html
	html div	id: 'tab_menu'; with: [
		html unorderedList with: [
			self tabsRoot keysAndValuesDo: [:section :subsections | 
				html listItem with: [ html text: section. html unorderedList with:[
					subsections keysDo: [:subsection |
						html listItem
							class: (self selectedTabKey = subsection ifTrue: ['active']);
							with: [html anchor callback: [self selectedTabKey: subsection; refreshTabs]; with: subsection]
					]
				]]
			]
		]
	]! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 11:54'!
selectedTabKey
	^ selectedTabKey! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 11:54'!
selectedTabKey: anObject 
	^ selectedTabKey := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
tabsRoot
	^ tabsRoot! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
tabsRoot: anObject
	tabsRoot := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'afk 1/6/2008 15:35'!
tagTabs
	^ tagTabs! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'afk 1/6/2008 15:35'!
tagTabs: anObject
	tagTabs := anObject! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
userTabs
	^ userTabs! !

!ITComponentAdmin methodsFor: 'accessing' stamp: 'mt 1/6/2008 17:23'!
userTabs: anObject
	userTabs := anObject! !

ITComponent subclass: #ITComponentAdminDetails
	instanceVariableNames: 'newTag newTagDescription newTags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAdminDetails methodsFor: 'private' stamp: 'afk 1/7/2008 19:18'!
applyChanges
	| session tag |
	
	ITSecurityManager checkPrivilege: 'add_issue'.
	
	session := ITGlorpSession getInstance.
	tag := ITTag with: self newTag with: self newTagDescription.
	
	session inUnitOfWorkDo: [
		session register: tag.
	].
	
	^tag.! !

!ITComponentAdminDetails methodsFor: 'private' stamp: 'afk 1/6/2008 23:41'!
deleteTagEntries
	self newTag: nil.
	self newTagDescription: nil.! !

!ITComponentAdminDetails methodsFor: 'private' stamp: 'afk 1/6/2008 16:49'!
deleteTagWithID: aNumber
	"Tries to delete a tag with the given ID from the database."
	
	| session tag |
	
	ITSecurityManager checkPrivilege: 'delete_issue'.
	
	session := ITGlorpSession getInstance.
	
	tag := session readOneOf: ITTag where: [:each | each id = aNumber].
	
	session delete: tag.
	! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTag
	^ newTag! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTag: anObject
	newTag := anObject! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:14'!
newTagDescription
	^ newTagDescription! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:14'!
newTagDescription: anObject
	newTagDescription := anObject! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTagValue
	^ newTagValue! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTagValue: anObject
	newTagValue := anObject! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTags
	^ newTags! !

!ITComponentAdminDetails methodsFor: 'accessing' stamp: 'afk 1/6/2008 14:00'!
newTags: anObject
	newTags := anObject! !

!ITComponentAdminDetails methodsFor: 'rendering' stamp: 'afk 1/6/2008 16:55'!
renderAddTagOn: html

	html fieldSet
		legend: 'Add a new tag:' tr;
		style: '	font-weight: normal;
				color: black;';
		with:[
			html form
				id: 'new_tag_form';
				with: 	[
					html table:[
						
						html tableHead with: [
							html tableRow with: [
								html tableHeading: 'Name'.
								html tableHeading: 'Description'.
							]
						].
						
						html tableRow: [
							html tableData: [ html textInput 
												script: html formElement focus;
												on: #newTag of: self. 
							].
							html tableData:  [ html textInput on: #newTagDescription of: self. ].
						html tableRow: [
							html tableData: [
								html submitButton 
									callback: [	
										[self applyChanges.]
											on: ITUnauthorizedError
											do: [self inform: 'You are not authorized to create a tag!!' tr].
										self deleteTagEntries.
										self answer: true. 
									];
									value: 'Add this tag' tr.
							].
						].
					]. 	
				].
			].
		].! !

!ITComponentAdminDetails methodsFor: 'rendering' stamp: 'afk 1/6/2008 15:24'!
renderContentOn: html
	"Renders the possibility to add new tags while creating a new Issue"
				
	self renderCurrentTagsOn: html.
	"self renderTagListsOn: html.	"	"Need apply button for that" "Function that not all tags muste be used"
	self renderAddTagOn: html.! !

!ITComponentAdminDetails methodsFor: 'rendering' stamp: 'afk 1/9/2008 19:02'!
renderCurrentTagsOn:  html
	"Shows all the tags that are currently in the database + abillity of deleting tags"
	
	| tags |
	tags := ITGlorpSession getInstance readManyOf: ITTag.
	
	html fieldSet
		legend: 'Currently available tags:' tr;
		style: '	font-weight: normal;
				color: black;';
		with: [ html table:[
					html tableHead with: [
					html tableRow: [
						html tableHeading: 'Name' tr.
						html tableHeading: 'Description' tr.
						html tableHeading: 'Option' tr.
					]]. 	
					html tableBody with: [
					tags do: [ :tag |
						
						html tableRow: [
							html tableData: [html strong: tag name].
							html tableData: tag description.
							html tableData: [
								html anchor
									callback: [ [(self deleteTagWithID: tag id)]
													on: ITUnauthorizedError
													do: [self inform: 'You are not authorized to delete a tag!!' tr].
									];
									with: 'delete' tr.
								]
						]. 
					]]
				]
		]! !

ITComponent subclass: #ITComponentAdminRoles
	instanceVariableNames: 'attachedPrivileges availablePrivileges selectedRole newRolename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mig 1/9/2008 17:18'!
addRole
	| newRole |
	newRole := ITRole new.
	newRole rolename: self newRolename.
	
	[self session securityManager addRole: newRole.]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to add a role!!' tr. ^self].
		
	self selectedRole: newRole.! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'afk 1/9/2008 16:13'!
applyChanges
	"Resets all privileges to the selected role."
	
	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: self selectedRole.
		self selectedRole privileges: self attachedPrivileges.
	]! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
attachedPrivileges
	^ attachedPrivileges! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
attachedPrivileges: anObject
	attachedPrivileges := anObject! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
availablePrivileges
	^ availablePrivileges! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 12/29/2007 15:12'!
availablePrivileges: anObject
	availablePrivileges := anObject! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mt 1/7/2008 23:03'!
deleteRole

	self selectedRole users ifNotEmpty: [
		self inform: 'You need to detach this role from all relevant users first before you can delete this one.'.
		^self
	].

	[self session securityManager deleteRoleWithID: self selectedRole id]
		on: ITUnauthorizedError
		do: [self inform: 'You are not authorized to delete a role!!' tr. ^self].
		
	self selectedRole: (ITGlorpSession getInstance readOneOf: ITRole).

	self updatePrivileges.! !

!ITComponentAdminRoles methodsFor: 'initialization' stamp: 'mt 1/5/2008 22:17'!
initialize
	super initialize.
	
	self newRolename: 'New Role' tr.
	
	selectedRole := ITGlorpSession getInstance readOneOf: ITRole.
	self updatePrivileges.! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:17'!
newRolename
	^ newRolename! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:17'!
newRolename: anObject
	newRolename := anObject! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 23:23'!
renderApplyChangesButtonOn: html
	html div
		class: 'button';
		style: 'float: right; margin: 10px 5px;';
		with: [
			html image url: ITFileLibrary / #acceptPng.
			html break.
			html anchor
				callback: [self applyChanges];
				with: [html text: 'Apply changes' tr]
		].! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/6/2008 00:35'!
renderAttachedUsersOn: html
	"Shows the current users which have the selected privilege attached."
	
	html fieldSet
		legend: 'Attached Users' tr, ' (', self selectedRole users size asString, ')';
		with: [
			html unorderedList
				with: [
					self selectedRole users do: [:user | html listItem with: user username]
				]
		]! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/6/2008 00:23'!
renderContentOn: html
	
	selectedRole isNil ifTrue: [selectedRole := ITGlorpSession getInstance readOneOf: ITRole].
	selectedRole isNil
			ifTrue: [
				html fieldSet
					legend: 'Add a new role' tr;
					with: [
						html span
							style: 'color: #000; font-weight: normal; ';
							with: 'There is no role available to be configured. You need to add a new one.' tr.
						html break.
						self renderNewRoleInputOn: html ].
				^nil.
			] ifFalse: [ self updatePrivileges ].

	html heading
		id: 'role_heading';
		level: 2;
		with: [ html text: 'Edit settings for role: ' tr, self selectedRole rolename ].
	
	html fieldSet
		legend: 'Select another role:' tr;
		with: [ self renderRoleSelectionOn: html ].
	
	html div
		id: 'privilege_lists';
		with: [ self renderPrivilegeListsOn: html ].
		
	html div
		id: 'attached_users';
		with: [ self renderAttachedUsersOn: html ].
	
	html fieldSet
		legend: 'Add a new role' tr;
		with: [ self renderNewRoleInputOn: html ].
	
	self renderDeleteRoleButtonOn: html.
	self renderApplyChangesButtonOn: html.! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 23:23'!
renderDeleteRoleButtonOn: html
	html div
		class: 'button';
		style: 'float: left; margin: 10px 5px';
		with: [
			html image url: ITFileLibrary / #deletePng.
			html break.
			html anchor
				callback: [self deleteRole];
				with: [html text: 'Delete Role' tr]
		].! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:50'!
renderListItems: aSelector on: html
	html unorderedList
		id: aSelector asString;
		script: (html sortable
			constraint: false;
			dropOnEmpty: true;
			containment: #( availablePrivileges attachedPrivileges );
			onUpdate: (html updater
				id: 'actual_privileges';
				triggerSortable: aSelector	callback: [ :values | self perform: aSelector asMutator with: values ]
			)
		);
		with: [
			(self perform: aSelector) do: [ :each |
				html listItem
					passenger: each;
					class: (aSelector = #availablePrivileges ifTrue: ['available'] ifFalse: ['attached']);
					with: each ] ]! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:27'!
renderNewRoleInputOn: html
	html form
		id: 'new_role_form';
		with: [
			html textInput on: #newRolename of: self.
			html submitButton 
				onClick: (html updater
					id: 'role_lists';
					triggerForm: 'new_role_form';
					callback: [:r | self addRole; updatePrivileges; renderPrivilegeListsOn: r]);
				value: 'add role' tr.
		].! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:34'!
renderPrivilegeListsOn: html

		html fieldSet
			class: 'drag_list';
			legend: 'Available Privileges' tr;
			with: [ self renderListItems: #availablePrivileges on: html ].
		html fieldSet
			class: 'drag_list';
			legend: 'Attached Privileges' tr;
			style: 'float: right;';
			with: [ self renderListItems: #attachedPrivileges on: html ].
		html div
			style: 'clear: both'.! !

!ITComponentAdminRoles methodsFor: 'rendering' stamp: 'afk 1/9/2008 16:16'!
renderRoleSelectionOn: html
	html form
		id: 'role_drop_down';
		with: [
			html select
				list: (ITGlorpSession getInstance read: ITRole);
				selected: self selectedRole;
				labels: [:role|role rolename];
				callback: [ :value | self selectedRole: value ];
				onChange: (html updater
					id: 'privilege_lists';
					triggerForm: 'role_drop_down';
					callback: [:r | self updatePrivileges; renderPrivilegeListsOn: r]);
				onChange: (html updater
					id: 'role_heading';
					triggerForm: 'role_drop_down';
					callback: [:r | r text: 'Edit settings for role: ' tr, self selectedRole rolename ]);
				onChange: (html updater
					id: 'attached_users';
					triggerForm: 'role_drop_down';
					callback: [:r | self renderAttachedUsersOn: r ]).										
		].! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:25'!
selectedRole
	^ selectedRole! !

!ITComponentAdminRoles methodsFor: 'accessing' stamp: 'mt 1/5/2008 22:25'!
selectedRole: anObject
	^ selectedRole := anObject! !

!ITComponentAdminRoles methodsFor: 'private' stamp: 'mig 1/7/2008 12:17'!
updatePrivileges
	"Updates the available and attached privileges to the current selected role."
	
	selectedRole ifNil: [^self].
	
	self attachedPrivileges: selectedRole privileges.
	self availablePrivileges:	(
		ITSecurityManager privileges asSet collect: [:each |
			(selectedRole privileges includes: each) not ifTrue: [each]
		]
	)
! !

ITComponent subclass: #ITComponentAdminUsers
	instanceVariableNames: 'attachedRoles availableRoles selectedUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentAdminUsers methodsFor: 'private' stamp: 'mt 1/7/2008 23:09'!
applyChanges
	"Resets all roles to the selected user and attaches the selected user to all roles that will be set."

	|session|
	session := ITGlorpSession getInstance.
	session inUnitOfWorkDo: [
		session register: self selectedUser.
		self selectedUser roles: self attachedRoles.

		self attachedRoles do: [:role | session register: role. role addUser: self selectedUser].
		self availableRoles do: [:role | session register: role. role removeUser: self selectedUser ]
	]! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
attachedRoles
	^ attachedRoles! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
attachedRoles: anObject
	attachedRoles := anObject! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
availableRoles
	^ availableRoles! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 12/29/2007 10:07'!
availableRoles: anObject
	availableRoles := anObject! !

!ITComponentAdminUsers methodsFor: 'initialization' stamp: 'mt 1/5/2008 22:17'!
initialize
	super initialize.
	
	selectedUser := self session securityManager getUser: '*DEFAULT'.
	self updateRoles.! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/5/2008 21:06'!
renderActualPrivilegesOn: html
	"Shows the current privileges the selected user would have if you apply/save the changes."
	
	| priv |
	priv := Set new.
	self attachedRoles do: [:role | priv := priv union: role privileges ].
	
	html fieldSet
		legend: 'Actual Privileges';
		with: [
			html unorderedList
				with: [
					priv do: [:privilege | html listItem with: privilege, ' ']
				]
		]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:55'!
renderApplyChangesButtonOn: html
	html div
		class: 'button';
		with: [
			html image url: ITFileLibrary / #acceptPng.
			html break.
			html anchor
				callback: [self applyChanges];
				with: [html text: 'Apply changes' tr]
		].! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:55'!
renderContentOn: html
	
	
	html heading
		id: 'user_heading';
		level: 2;
		with: [ html text: 'Edit security settings for: ' tr, selectedUser username ].
	
	html fieldSet
		legend: 'Select another user:' tr;
		with: [ self renderUserSelectionOn: html ].
	
	html div
		id: 'role_lists';
		with: [ self renderRoleListsOn: html ].
		
	html div
		id: 'actual_privileges';
		with: [ self renderActualPrivilegesOn: html ].
		
	self renderApplyChangesButtonOn: html.! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/5/2008 22:53'!
renderListItems: aSelector on: html
	html unorderedList
		id: aSelector asString;
		script: (html sortable
			constraint: false;
			dropOnEmpty: true;
			containment: #( availableRoles attachedRoles );
			onUpdate: (html updater
				id: 'actual_privileges';
				triggerSortable: aSelector	callback: [ :values | self perform: aSelector asMutator with: values ];
				callback: [ :renderer | self renderActualPrivilegesOn: renderer ]
			)
		);
		with: [
			(self perform: aSelector) do: [ :each |
				html listItem
					passenger: each;
					class: (aSelector = #availableRoles ifTrue: ['available'] ifFalse: ['attached']);
					with: each rolename tr] ]! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/5/2008 21:10'!
renderRoleListsOn: html
		html fieldSet
			class: 'drag_list';
			legend: 'Available Roles';
			with: [ self renderListItems: #availableRoles on: html ].
		html fieldSet
			class: 'drag_list';
			legend: 'Attached Roles';
			style: 'float: right;';
			with: [ self renderListItems: #attachedRoles on: html ].
		html div
			style: 'clear: both'.! !

!ITComponentAdminUsers methodsFor: 'rendering' stamp: 'mt 1/6/2008 00:27'!
renderUserSelectionOn: html
	html form
		id: 'user_drop_down';
		with: [
			html select
				list: (ITGlorpSession getInstance read: ITUser);
				selected: selectedUser;
				labels: [:user|user username, ' (', user id asString, ')'];
				callback: [ :value | selectedUser := value ];
				onChange: (html updater
					id: 'role_lists';
					triggerForm: 'user_drop_down';
					callback: [:r | self updateRoles; renderRoleListsOn: r]);
				onChange: (html updater
					id: 'user_heading';
					triggerForm: 'user_drop_down';
					callback: [:r | r text: 'Edit security settings for: ' tr, selectedUser username]);
				onChange: (html updater
					id: 'actual_privileges';
					triggerForm: 'user_drop_down';
					callback: [:r | self renderActualPrivilegesOn: r ]).					
		]! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 1/6/2008 00:43'!
selectedUser
	^ selectedUser! !

!ITComponentAdminUsers methodsFor: 'accessing' stamp: 'mt 1/6/2008 00:43'!
selectedUser: anObject
	^ selectedUser := anObject! !

!ITComponentAdminUsers methodsFor: 'private' stamp: 'mt 12/29/2007 12:44'!
updateRoles
	"Updates the available and attached roles to the current selected User."
	
	self attachedRoles: selectedUser roles.
	self availableRoles:	(
		(ITGlorpSession getInstance read: ITRole) asSet collect: [:each |
			(selectedUser roles includes: each) not ifTrue: [each]
		]
	).! !

ITComponent subclass: #ITComponentDetails
	instanceVariableNames: 'currentIssue addComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentDetails commentStamp: 'mig 1/9/2008 12:00' prior: 0!
I show all the details for an issue and provide an interface to change and append things.!

!ITComponentDetails class methodsFor: 'instance creation' stamp: 'mt 1/5/2008 10:33'!
newWith: anID
	|view|
	view := self new.
	
	[
	view currentIssue: (view session issueManager getIssueWithID: anID).
	view addComment: (ITComponentAddComment newWith: anID).
	]
		on: ITUnauthorizedError
		do: [view currentIssue: nil; addComment: nil].
		
	
	^view.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:36'!
addComment
	^ addComment! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:36'!
addComment: anObject
	addComment := anObject! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 23:41'!
children
	^ Array with: self addComment.! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 22:40'!
currentIssue
	^ currentIssue! !

!ITComponentDetails methodsFor: 'accessing' stamp: 'mt 1/4/2008 22:40'!
currentIssue: anObject
	currentIssue := anObject! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mig 1/9/2008 18:21'!
renderCommentsOn: html
	
	(self currentIssue comments) do: [:comment |
		html div
			class: 'comment';
			with: [ 
				html heading
					level: 3;
					with: ('{1} by {2}' tr format: {comment timestamp asString. comment user username}).
				html text: comment content.
			]
	]! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mig 1/9/2008 18:14'!
renderContentOn: html
	"Show details of the current issue."

	html div
		class: 'content';
		with: [
			self currentIssue isNil ifFalse: [
				html heading
					with: ('Issue #{1}' tr format: self currentIssue id asString);
					level: 2.
				
				self renderDetailsOn: html.
				
				html heading
					with: 'Comments:' tr;
					level: 2.
					
				self renderCommentsOn: html.

				html render: self addComment.
				
			] ifTrue: [
				html break.
				html text: 'You are not authorized to view the details of an issue!!' tr.
			].
			
			
			5 timesRepeat: [html break]
		]
				! !

!ITComponentDetails methodsFor: 'rendering' stamp: 'mt 1/8/2008 21:21'!
renderDetailsOn: html

	html div
		id: 'issue';
		with: [
			html heading: (self currentIssue getTag: 'title') level: 2.
			html table
				class: 'details';
				with: [
					html tableRow: [ 
						html tableHeading: 'Created by:' tr.
						html tableData: 'someBody'.
						html tableHeading: 'Assigned to:' tr.
						html tableData: 'aDeveloper'].
					html tableRow: [ 
						html tableHeading: 'Priority:' tr.
						html tableData: 'urgent'.
						html tableHeading: 'Version:' tr.
						html tableData: '1.0.0'].
					html tableRow: [ 
						html tableHeading: 'Color:' tr.
						html tableData: 'yellow'.
						html tableHeading: 'another tag:' tr.
						html tableData: 'another value'].
				].
			html div
				class: 'description';
				with: [
					self currentIssue details do: [:each | 
													each tag = 'title' 	ifTrue: ["html heading: (each content asString) level: 2."]
																		ifFalse: [	
																				html heading: (each tag asString) level: 3.
																				html paragraph: (each content asString).
																				].
												].
				]
		]! !

ITComponent subclass: #ITComponentListIssues
	instanceVariableNames: 'currentIssues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentListIssues commentStamp: 'mig 1/9/2008 12:00' prior: 0!
I list all issues in the issue tracker.!

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mt 1/6/2008 12:15'!
currentIssues
	^ currentIssues! !

!ITComponentListIssues methodsFor: 'accessing' stamp: 'mt 1/6/2008 12:15'!
currentIssues: anObject
	currentIssues := anObject! !

!ITComponentListIssues methodsFor: 'private' stamp: 'mt 1/6/2008 12:17'!
refreshCurrentIssues
	"Retrieves all available issues through the issue manager from the database."
	
	self currentIssues: self session issueManager getIssues! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'mt 1/6/2008 14:07'!
renderContentOn: html

	[self refreshCurrentIssues]
		on: ITUnauthorizedError
		do: [html text: 'You are not authorized to list all issues!!' tr. ^nil].

	html heading 
		level: 2;
		with: 'All Issues' tr, ' (', self currentIssues size asString, ')'.
		
	html div
		with: [self renderIssueTableOn: html].! !

!ITComponentListIssues methodsFor: 'rendering' stamp: 'mig 1/9/2008 18:17'!
renderIssueTableOn: html
	"Renders a table with necessary information of all available issues. There is a link under the name of every issue which displays a detailed view. See ITComponentDetails for more information."
	
	| class |
	class := 'even'.
	
	html table
		class: 'list_issues';
		style: 'padding: 0;';
		with: [
			html tableHead with: [
			html tableRow: [
				html tableHeading with: '#'.
				html tableHeading with: 'Title' tr.
				html tableHeading with: 'Created' tr.
				html tableHeading with: 'Owner' tr.
				html tableHeading with: [html image url: ITFileLibrary / #commentsPng].
				html tableHeading with: 'Last Comment' tr.
				html tableHeading with: 'From' tr.
			]].
			html tableBody with: [
			self currentIssues do: [:issue| |callback|
				callback := [self call: (ITComponentDetails newWith: issue id)].
				html tableRow
					class: (class = 'odd' ifTrue: [class := 'even'] ifFalse: [class := 'odd']);
					with: [
					html tableData with: [
						html anchor
							title: 'View Details' tr;
							callback: callback;
							with: [html text: issue id asString]].				
					html tableData	with: [
						html anchor
							title: 'View Details' tr;
							callback: callback;
							with: [html text: (issue getTag: 'title')]].
					"html tableData: [
						html anchor
							callback: [
								[((self session issueManager) deleteIssueWithID: issue id)]
									on: ITUnauthorizedError
									do: [self inform: 'You are not authorized to delete an issue!!' tr]
							];
							with: [html image url: ITFileLibrary / #binPng; altText: 'delete' tr ].
						]."
					
					html tableData with: (issue firstComment printTimeStamp).
					html tableData with: (issue firstComment user username).
					html tableData with: (issue comments size).
					html tableData with: (issue latestComment printTimeStamp).
					html tableData with: (issue latestComment user username).					
					].
				].
			].
		]
! !

ITComponent subclass: #ITComponentLogin
	instanceVariableNames: 'userInput passwordInput'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentLogin commentStamp: 'mig 1/9/2008 12:01' prior: 0!
I am the component where the user can login.!

!ITComponentLogin methodsFor: 'private protocol' stamp: 'afk 1/8/2008 13:01'!
login
	"Logic for logging in a user"
	| loginSucceeded |
	
	loginSucceeded := self session securityManager loginUser: self userInput withPassword: self passwordInput. 
	loginSucceeded 
		ifTrue: [self answer: true]
		ifFalse: [self inform: 'Wrong username/password combination!!'.]! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput
	^ passwordInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
passwordInput: anObject
	passwordInput := anObject! !

!ITComponentLogin methodsFor: 'rendering' stamp: 'afk 1/8/2008 00:18'!
renderContentOn: html

	html div style: 'background: white; padding: 20px;'; with: [
		html form: [
			html text: 'username:'.
			html textInput on: #userInput of: self.
			html break.
			html text: 'password:'.
			html passwordInput on: #passwordInput of: self.
			html break.
			html submitButton callback: [
				self login.
			]; value: 'Login'.
			html cancelButton callback: [
				self answer: false.
			]; value: 'Cancel'.
		].
	].! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput
	^ userInput! !

!ITComponentLogin methodsFor: 'accessing' stamp: 'mig 12/17/2007 13:42'!
userInput: anObject
	userInput := anObject! !

ITComponent subclass: #ITComponentMenu
	instanceVariableNames: 'tabs priviligeDict userStatus selectedTabIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentMenu class methodsFor: 'testing' stamp: 'mt 11/23/2007 18:52'!
canBeRoot
	^true.! !

!ITComponentMenu methodsFor: 'private' stamp: 'afk 1/9/2008 17:57'!
checkPrivilegeForTab: assoc withPriv: priv

	[ITSecurityManager checkPrivilege: priv.]
		on: ITUnauthorizedError
		do: [^false].

 	^ true.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 1/6/2008 20:53'!
children
	^Array with: (self tabs at: self selectedTabIndex) value with: self userStatus.! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mfp 1/8/2008 16:34'!
initialRequest: aRequest
	"ggf. Tabwechsel auslagern"
	| inputFields |
	super initialRequest: aRequest.
	inputFields := aRequest fields.
	inputFields at: 'component' ifPresent: [:componentNumber |
		self switchTabTo: componentNumber asNumber.
		].
	
		"singleField in tabs ifTrue:[
			Transcript show: singleField.
			self 
				selectedTabIndex: singleField;
				refreshTabs.
		]."
	! !

!ITComponentMenu methodsFor: 'initialize' stamp: 'mt 1/6/2008 20:58'!
initialize
	super initialize.
	self userStatus: ITComponentUserStatus new.
	self refreshTabs.
	
	self selectedTabIndex: 1.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'afk 1/9/2008 17:27'!
priviligeDict
	^ priviligeDict! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'afk 1/9/2008 17:27'!
priviligeDict: anObject
	priviligeDict := anObject! !

!ITComponentMenu methodsFor: 'private' stamp: 'afk 1/9/2008 17:43'!
refreshTabs
	"Creates new component objects for the children but keeps the current selected tab."
	
	self tabs: (OrderedCollection new
		add: 'View Issues' -> (ITComponentListIssues new parent: self; yourself);
		add: 'New Issue' -> (ITComponentAddIssue new parent: self; yourself);
		add: 'Administration' -> (ITComponentAdmin new parent: self; yourself);
		yourself).
		
	"this could be moved to initialize (single method) but here you can see the needed equivalence between
	tabs and priviligeDict"
	self priviligeDict: (Dictionary new
		at: 'View Issues' put: #'read_issue';
		at: 'New Issue' put: #'add_issue';
		at: 'Administration' put: #'administrate'; yourself)! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'afk 1/8/2008 22:11'!
renderChildrenOn: html
	html div
		class: 'content';
		with: [
			html render: (self tabs at: self selectedTabIndex) value.
		]! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'afk 1/9/2008 12:44'!
renderContentOn: html

	html div
		id: 'top';
		with: [
			html div
				id: 'project_name';
				with: [html heading: 'Projectname' level:1 ].
		
			html div
				id: 'login_menu';
				with: [self renderLoginMenuOn: html].	
		].
	
	self renderMainMenuOn: html.
	self renderChildrenOn: html.
	self renderFooterOn: html.
	! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mt 1/9/2008 08:36'!
renderFooterOn: html
	"Display the footer on every page after the content"
	
	html div
		class: 'footer';
		with: [
		
			html div
				class: 'footer_right';
				with: [
					html 	
						text: 'Visit yaiiit!! project page on'; 
						break; 
						withUrls: 'http://www.yaiiit.net'
				].
			
			html div
				with: [
					html image url: ITFileLibrary / #logobottomPng.
				].			
		].! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'mig 1/9/2008 18:02'!
renderLoginMenuOn: html
	html span with: [
		html render: self userStatus.
		"html text: ' | '."
	].
! !

!ITComponentMenu methodsFor: 'rendering' stamp: 'afk 1/9/2008 17:44'!
renderMainMenuOn: html

	html div id: 'main_menu'; with: [
		html unorderedList with: [
			self tabs withIndexDo: [ :ass :index |
				(self checkPrivilegeForTab: ass withPriv: (self priviligeDict at: ass key))
					ifTrue: [
						html listItem
							class: (self selectedTabIndex = index ifTrue: ['active']);
							with: [ 
								html anchor 
									callback: [ self switchTabTo: index];
									with: ass key
							]
					]
		]   
	] ]! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 1/6/2008 20:51'!
selectedTabIndex
	^ selectedTabIndex! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mt 1/6/2008 20:51'!
selectedTabIndex: anObject 
	^ selectedTabIndex := anObject! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mfp 1/8/2008 16:33'!
switchTabTo: anIndex

	self
		selectedTabIndex: anIndex;
		refreshTabs.! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'afk 12/30/2007 22:32'!
tabs
	^ tabs! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'afk 12/30/2007 22:32'!
tabs: anObject
	tabs := anObject! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:36'!
userStatus
	^ userStatus! !

!ITComponentMenu methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:36'!
userStatus: anObject
	userStatus := anObject! !

ITComponent subclass: #ITComponentMessage
	instanceVariableNames: 'title message buttons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
buttons
	^ buttons! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
buttons: anObject
	buttons := anObject! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
message
	^ message! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
message: anObject
	message := anObject! !

!ITComponentMessage methodsFor: 'rendering' stamp: 'mig 1/9/2008 17:17'!
renderContentOn: html
	html div
		class: 'messagebox';
		with: [
			html form with: [
				html div class: 'title'; with: self title.
				html div class: 'message'; with: self message.
				html div class: 'buttons'; with: [
					self buttons withIndexDo: [:button :index |
						html submitButton 
							value: button;
							callback: [self answer: index.].
					]
				]
			]
		]! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
title
	^ title! !

!ITComponentMessage methodsFor: 'accessing' stamp: 'mig 1/9/2008 16:56'!
title: anObject
	title := anObject! !

ITComponent subclass: #ITComponentUserRegister
	instanceVariableNames: 'userInput passwordInput1 passwordInput2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentUserRegister methodsFor: 'as yet unclassified' stamp: 'afk 1/8/2008 00:25'!
loginAfterRegister

	| login |
	login := ITComponentLogin new.
	login userInput: self userInput.
	login passwordInput: self passwordInput1.
	login login.! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1
	^ passwordInput1! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput1: anObject
	passwordInput1 := anObject! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2
	^ passwordInput2! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
passwordInput2: anObject
	passwordInput2 := anObject! !

!ITComponentUserRegister methodsFor: 'rendering' stamp: 'mig 1/9/2008 16:40'!
registerUser
	"trys to register the user, but only if passwords match and user does not already exists; logs in if successful"

	| user |
	
	self passwordInput1 = self passwordInput2 ifFalse: [
		self inform: 'The passwords you entered do not match.' tr.
		^self.
	].

	user := ITUser new.
	user
		username: self userInput;
		setPassword: self passwordInput1.
			
	[self session securityManager addUser: user]
		on: ITUserAlreadyExists 
		do: [
			self inform: ('The user {1} already exists.' tr format: {user username}).
			^self.
		].

	self loginAfterRegister.
	self answer: true.! !

!ITComponentUserRegister methodsFor: 'rendering' stamp: 'mig 1/9/2008 16:29'!
renderContentOn: html
	html div style: 'background: white; padding: 20px;'; with: [
		html form: [
			html text: 'username:'.
			html textInput on: #userInput of: self.
			html break.
			html text: 'password:'.
			html passwordInput on: #passwordInput1 of: self.
			html break.
			html text: 'verify:'.
			html passwordInput on: #passwordInput2 of: self.
			html break.
			html submitButton 
				callback: [ self registerUser. ]; 
				value: 'Register'.
			html cancelButton 
				callback: [ self answer: false. ]; 
				value: 'Cancel'.
		].
	].! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput
	^ userInput! !

!ITComponentUserRegister methodsFor: 'accessing' stamp: 'mig 12/17/2007 14:00'!
userInput: anObject
	userInput := anObject! !

ITComponent subclass: #ITComponentUserStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITComponentUserStatus commentStamp: 'mig 1/9/2008 12:02' prior: 0!
I show whether an user is logged in and provide the corresponding login or logout button.!

!ITComponentUserStatus methodsFor: 'rendering' stamp: 'mig 1/9/2008 17:19'!
renderContentOn: html
	self session isLoggedIn ifFalse: [
		html anchor
			callback: [ self lightbox: ITComponentLogin new ]; 
			with: [ html text: 'Login' tr].
		html text: ' | '.
		html anchor 
			callback: [ self lightbox: ITComponentUserRegister new ]; 
			with: [ html text: 'Register' tr].
	] ifTrue: [
		html span
			class: 'text';
			with: ('logged in as {1}' tr format: {self session user username}).
		html text: ' | '.
		html anchor
			callback: [self session user: nil.];
			with: 'Logout' tr.
	].! !

Error subclass: #ITError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITError commentStamp: 'mt 12/30/2007 14:39' prior: 0!
I am an generic error which should only be signaled in this application.!

ITError subclass: #ITUnauthorizedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!

!ITUnauthorizedError commentStamp: 'mt 1/8/2008 12:14' prior: 0!
An ITUnauthorizedError is thrown, when a user has not sufficient privileges to perform an action.

I should be signaled by the security manager.!

ITError subclass: #ITUserAlreadyExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SWE-IssueTracker'!
